# import wpkit
# from wpkit.web import bps
# app=wpkit.web.get_default_app(__name__)
# app.register_blueprint(bps.bp_pan.bp_pan(app,url_prefix='/pan'))


from wpkit.node import *
from wpkit.node import components
from wpkit.node.Q import *
from wpkit.node.components import *

# body=Body(style="background-color:#ffffff;")(
#     Div(_class="row",style="width:300px;height:300px;background-color:#fafafa"),
#     Div(_class="row",style="width:300px;height:300px;background-color:yellow;")
# )
post_script = Text("var data={cmd:{op:1,params:[2]}};console.log(data);res=postJson('/pan/data',data);console.log(res)")
body = Div(style=StyleAttr(height="100%", width="100%"))(
    QBox().css(height='{{h1}}%'),
    QBox()(
        QBox(style=StyleAttr(float="left", width="calc({{w}}%)", background_color="#eeeeee"))(
            QButton(onclick=post_script)("Click Me")
        ),
        QBox(id=3)(
            QCell()("Hello")
        ).css(float="right", width="calc({{100-w}}%)")
    ).css(height='{{100-h1}}%')
)
from wpkit.web import resources
body = Div(_class='container')(
    link.csslink(href='/pkg-resource/css/jquery-resizable.css'),
    Script(src='/pkg-resource/js/jquery-resizable.js'),
    Script()(
        resources.get_js_string_by_name('my.js')
    ),
    Script(src="/pkg-resource/pkgs/vue-grid-layout/vue-grid-layout.umd.min.js"),
    Script(src='/pkg-resource/pkgs/vue-draggable-resizable/vue-draggable-resizable.js')(),
    link.csslink(href='/pkg-resource/pkgs/vue-draggable-resizable/vue-draggable-resizable.css'),
    Script()(
        '''
        function getDir(location,dirname){
                var cmd={cmd:{op:"getDir",params:{location:location,dirname:dirname}}};
        console.log(cmd);var res=postJson('/pan/cmd',cmd).responseJSON;console.log(res);
        return res;
            }
        function getFile(location,filename){
            var cmd={cmd:{op:"getFile",params:{location:location,filename:filename}}};
        console.log(cmd);var res=postJson('/pan/cmd',cmd).responseJSON;console.log(res);
        return res;
        }
        $(document).ready(()=>{
            
        })
        '''
    ),
    Div()(
        resources.get_template_string_by_name('tmp')
    ),
    QRow()(
        resources.get_template_string_by_name('test')
    )
)
panpage = components.Sitebase().render(body=body)
# testobj=body
# panpage["body"].css(padding=0, margin=0,background="black",color="white")
panpage["body"].css(padding=0, margin=0)
testobj = body
panpage = panpage.compile().render(h1=8, w=8)




from wpkit.piu import BackupDB
from flask import Flask,jsonify
from wpkit.web.utils import parse_json,parse_json_and_form
from wpkit.basic import PointDict,Status,StatusSuccess,StatusError
from wpkit.web.base import MyBlueprint,Application
from wpkit.web import utils,resources

class DBServer(MyBlueprint):
    def __init__(self,import_name='__main__',dbpath="./data/db",url_prefix='/db',nickname="DBServer",*args,**kwargs):
        self.db = BackupDB(path=dbpath)
        super().__init__(import_name=import_name,url_prefix=url_prefix,nickname=nickname,enable_CORS=True,add_to_sitemap=False,*args,**kwargs)
    def add_handlers(self):
        self.add_static(url_prefix='/', static_dir=self.db.dbpath)
        @self.route('/cmd',methods=['POST','GET'])
        @parse_json_and_form
        def do_cmd(cmd):
            print("cmd:",cmd)
            try:
                res=self.db.execute(cmd)
                res=StatusSuccess(data=res)
            except:
                res=StatusError()
            return jsonify(res)
        @self.route('/board',methods=['GET','POST'])
        def do_board():
            return resources.get_template_by_name('dbboard').render()
if __name__ == '__main__':
    DBServer(__name__).run()








import time,datetime
def to_chinese_date(t):
    date=datetime.datetime.fromtimestamp(t)
    date=date.strftime('%Y{y}%m{m}%d{d}').format(y='年', m='月', d='日')
    return date
def get_time_formated(format='%Y-%m-%d %H:%M:%S'):
    import time
    return time.strftime(format, time.localtime())


def generate_hash(s, times=1):
    assert times >= 1
    import hashlib
    m = hashlib.md5()

    def gen():
        m.update(s.encode('utf-8'))
        return m.hexdigest()[:10]

    for i in range(times):
        data = gen()
    return data


def is_all_chinese(strs):
    for _char in strs:
        if not '\u4e00' <= _char <= '\u9fa5':
            return False
    return True


def is_contains_chinese(strs):
    for _char in strs:
        if '\u4e00' <= _char <= '\u9fa5':
            return True
    return False


def is_all_alphabet(text=''):
    for char in text:
        if not char.isalpha():
            return False
    return True


def is_all_alphabet_or_digit(text=''):
    for char in text:
        if not char.isalpha() and not char.isdigit():
            return False
    return True



'''
Deprecated
'''

from dulwich import porcelain as git
from dulwich.repo import Repo
from wk import FakeOS
from wk import T
import os, shutil, glob

default_remote_location= 'https://OpenGitspace:Gitspace@123456@gitee.com/OpenGitspace/meta'
defalut_user_location='https://OpenGitspace:Gitspace@123456@gitee.com/OpenGitspace/'

def get_default_remote_location(repo):
    return defalut_user_location+repo

class errors:
    class GitSpaceError(Exception):
        def __init__(self, *args):
            super().__init__(*args)


def is_git_dir(path):
    if os.path.exists(path) and os.path.isdir(path):
        dot_git_dir = path + '/.git'
        if os.path.exists(dot_git_dir) and os.path.isdir(dot_git_dir):
            return True
    return False


def is_empty_dir(path):
    assert os.path.exists(path) and os.path.isdir(path)
    if len(os.listdir(path)):
        return False
    else:
        return True

def to_string_iterable(iterable):
    lis=[s.decode() if isinstance(s,bytes) else s for s in iterable]
    return iterable.__class__(lis)
class GitRepo(Repo):
    def __init__(self,path):
        super().__init__(path)

    def clean(self):
        for name in os.listdir(self.path):
            if name=='.git':continue
            path=self.path+'/'+name
            if os.path.isdir(path):
                shutil.rmtree(path)
            else:
                os.remove(path)
    def active_branch(self):
        return git.active_branch(self).decode()
    def checkout_branch(self,branch='master'):
        git.update_head(self,branch)
        self.clean()
        co_ref = b'HEAD'
        repo_path=self.path
        from dulwich.repo import Repo
        from dulwich.index import build_index_from_tree
        repo = Repo(repo_path)
        indexfile = repo.index_path()
        obj_sto = repo.object_store
        tree_id = repo[co_ref].tree
        build_index_from_tree(repo_path, indexfile, obj_sto, tree_id)
        x=list(obj_sto.iter_tree_contents(tree_id))
        x=[obj_sto.iter_tree_contents(tree_id)]
    def branch_create(self,name):
        git.branch_create(self,name)
    def pull(self, remote_location, branch):
        git.pull(self, remote_location, branch)
    def branch_list(self):
        return list(to_string_iterable(git.branch_list(self)))
    def status(self,silent=False):
        repo = self
        msg = git.status(repo)
        if not silent:
            print(msg)
        return msg
    def add_all(self):
        repo = self
        status=git.status(self)
        paths = status.untracked + status.unstaged
        paths = [p.decode() if isinstance(p, bytes) else p for p in paths]
        paths = [self.path + '/' + p for p in paths]
        if not paths:
            import logging
            logging.warning('Working tree is clean. Nothing to add.')
            return
        paths.append(self.path)
        git.add(repo, paths)
    def commit(self,msg='commit somthing'):
        git.commit(self, msg)
    def push(self, remote_location, branch):
        git.push(self, remote_location, branch)



class GitRepoPlus:
    def __init__(self,path):
        self.grepo=GitRepo(path)
    def _init_branches(self):
        grepo=self.grepo
        # try:
            # grepo.pull()
    def _true_branch_name(self,name):
        import uuid
    def clean(self):
        for name in os.listdir(self.path):
            if name=='.git':continue
            path=self.path+'/'+name
            if os.path.isdir(path):
                shutil.rmtree(path)
            else:
                os.remove(path)
    def active_branch(self):
        return git.active_branch(self).decode()
    def checkout_branch(self,branch='master'):
        git.update_head(self,branch)
        self.clean()
        co_ref = b'HEAD'
        repo_path=self.path
        from dulwich.repo import Repo
        from dulwich.index import build_index_from_tree
        repo = Repo(repo_path)
        indexfile = repo.index_path()
        obj_sto = repo.object_store
        tree_id = repo[co_ref].tree
        build_index_from_tree(repo_path, indexfile, obj_sto, tree_id)
        x=list(obj_sto.iter_tree_contents(tree_id))
        x=[obj_sto.iter_tree_contents(tree_id)]
    def branch_create(self,name):
        git.branch_create(self,name)
    def pull(self, remote_location, branch):
        git.pull(self, remote_location, branch)
    def branch_list(self):
        return list(to_string_iterable(git.branch_list(self)))
    def status(self,silent=False):
        repo = self
        msg = git.status(repo)
        if not silent:
            print(msg)
        return msg
    def add_all(self):
        repo = self
        status=git.status(self)
        paths = status.untracked + status.unstaged
        paths = [p.decode() if isinstance(p, bytes) else p for p in paths]
        paths = [self.path + '/' + p for p in paths]
        if not paths:
            import logging
            logging.warning('Working tree is clean. Nothing to add.')
            return
        paths.append(self.path)
        git.add(repo, paths)
    def commit(self,msg='commit somthing'):
        git.commit(self, msg)
    def push(self, remote_location, branch):
        git.push(self, remote_location, branch)

def clone(src, path=None, overwrite=False,branch='master'):
    path = path or os.path.basename(src).rsplit('.git', maxsplit=1)[0]
    if os.path.exists(path) and overwrite:
        shutil.rmtree(path)
    assert not os.path.exists(path) or is_empty_dir(path)
    git.clone(src, path)
    repo=GitRepo(path)
    if not branch=='master':
        repo.pull(remote_location=src,branch=branch)
    return repo




class GitSpaceRepo:
    def __init__(self, path, remote_location=None):
        self.repo = Repo(path)
        self.path = os.path.abspath(path)
        self.remote_location = remote_location
        if not 'empty' in self.branch_list():
            self.branch_create('empty')
            br=self.active_branch()
            self.checkout_branch('empty')
            self.clean()
            self.stage()
            git.commit(self.repo,'empty')
            self.checkout_branch(br)
        self.git=git
    def all_remote_branches(self):
        raise NotImplementedError('')
    def log_new_branch(self,branch):
        raise NotImplementedError('')
    def clean(self):
        for name in os.listdir(self.path):
            if name=='.git':continue
            path=self.path+'/'+name
            if os.path.isdir(path):
                shutil.rmtree(path)
            else:
                os.remove(path)
    def active_branch(self):
        return git.active_branch(self.repo).decode()
    def checkout_branch(self,branch='master'):
        git.update_head(self.repo,branch)
        self.clean()
        co_ref = b'HEAD'
        repo_path=self.path
        from dulwich.repo import Repo
        from dulwich.index import build_index_from_tree
        repo = Repo(repo_path)
        indexfile = repo.index_path()
        obj_sto = repo.object_store
        tree_id = repo[co_ref].tree
        build_index_from_tree(repo_path, indexfile, obj_sto, tree_id)
        x=list(obj_sto.iter_tree_contents(tree_id))
        # print('branch contents:',x)
        x=[obj_sto.iter_tree_contents(tree_id)]
        # self.clean()
    def branch_create_empty(self, name):
        br=self.active_branch()
        self.checkout_branch('empty')
        self.branch_create(name)
        self.checkout_branch(br)
    def branch_create(self,name):
        repo=self.repo
        git.branch_create(repo,name)
    def pull(self, remote_location=None, branch='master'):
        remote_location = remote_location or self.remote_location
        if not remote_location:
            raise Exception('Remote location is not given.')
        repo = self.repo
        git.pull(repo, remote_location, branch)
    def branch_list(self):
        return list(to_string_iterable(git.branch_list(self.repo)))
    def status(self,silent=False):
        repo = self.repo
        msg = git.status(repo)
        if not silent:
            print(msg)
        return msg
    def stage(self):
        repo = self.repo
        status=git.status(self.repo)
        paths = status.untracked + status.unstaged
        paths = [p.decode() if isinstance(p, bytes) else p for p in paths]
        paths = [self.path + '/' + p for p in paths]
        if not paths:
            import logging
            logging.warning('Working tree is clean. Nothing to add.')
            return
        paths.append(self.path)
        git.add(repo, paths)
    def push(self, remote_location=None, branch=None):
        branch=branch or self.active_branch() or 'master'
        # print("branch_to_push:",branch)
        remote_location = remote_location or self.remote_location
        if not remote_location:
            raise Exception('Remote location is not given.')
        repo = self.repo
        self.stage()
        git.commit(repo, 'bare gitspace commit')
        git.push(repo, remote_location, branch)

    @classmethod
    def init(cls, path):
        git.init(path)
        return cls(path)

    @classmethod
    def clone(cls, src, path=None, overwrite=False,branch='master'):
        path = path or os.path.basename(src).rsplit('.git', maxsplit=1)[0]
        if os.path.exists(path) and overwrite:
            shutil.rmtree(path)
        assert not os.path.exists(path) or is_empty_dir(path)
        git.clone(src, path)
        repo=cls(path, src)
        repo.pull(branch=branch)
        return repo

    @classmethod
    def openSpace(cls, path, remote_path=None,branch='master'):
        if not os.path.exists(path) or is_empty_dir(path):
            assert remote_path
            cls.clone(remote_path, path,branch=branch)
            repo = cls(path, remote_path)
            return repo
        elif is_git_dir(path):
            return cls(path, remote_path)
        else:
            raise Exception('A non-empty directory %s already exists and it is not a git repo.' % (path))


class GitSpace(GitSpaceRepo, FakeOS):
    def __init__(self, path, remote_location=None):
        GitSpaceRepo.__init__(self, path, remote_location)
        FakeOS.__init__(self, path)


def open_default(path,branch='master'):
    # url = 'https://OpenGitspace:Gitspace@123456@github.com/OpenGitspace/meta'
    url =default_remote_location
    return GitSpace.openSpace(path, remote_path=url,branch=branch)

class SimpleStore:
    def __init__(self,remote_location=None,cache_dir=None):
        self.remote_location=remote_location or default_remote_location
        self.cache_dir=cache_dir or '.store_cache'
        self._setup()
    def _get_config(self):
        cfg = self.space.openFiledict('.store.cfg')
        return cfg
    def _setup(self):
        space=GitSpace.openSpace(path=self.cache_dir,remote_path=self.remote_location,branch='master')
        cfg=space.openFiledict('.store.cfg')
        if not cfg.get('store_keys'):
            cfg.store_keys=['master','empty']
            space.push()
        self.space=space
    def _add_key_to_config(self,key):
        space=self.space
        cfg = self._get_config()
        if key in cfg.store_keys:return
        cfg.store_keys.append(key)
        cfg._save()
        self.space.push()
    def keys(self):
        cfg=self._get_config()
        # print(cfg)
        # input()
        return cfg.store_keys
    def status(self):
        from wk import PointDict
        info=PointDict(
            current_branch=self.space.active_branch(),
            branches=self.space.branch_list(),
            status=self.space.status(silent=True)
        )
        if self.space.active_branch()=='master':
            info.store_keys=self.keys()
        # info.print()
        print(info)
        return info
    def store(self,key,path,overwrite=False):
        assert os.path.exists(path)
        br=self.space.active_branch()
        space=self.space
        if key in self.keys():
            if not overwrite:
                raise Exception('Key "%s" already exists.'%(key))
            # self.space.pull(branch=key)
            # self.space.clean()
        if key in space.branch_list():
            self._add_key_to_config(key)
            space.checkout_branch(key)
            space.clean()
        else:
            self.space.branch_create_empty(name=key)
            self._add_key_to_config(key)
            self.space.checkout_branch(key)
        # self.status()
        self._copy_to_cache_dir(path)
        # self.status()

        self.space.push()
        # self.status()
        # print('br:',br)
        self.space.checkout_branch(br)
        # self.status()
        # input()
        # self.status()

    def fetch(self,key,dst,default=T.NOT_GIVEN):
        space=self.space
        br=space.active_branch()
        # self.status()
        if key in space.branch_list():
            # use local branch
            space.checkout_branch(key)
        else:
            # pull from remote
            if not key in self.keys():
                if T.NOT_GIVEN(default):
                    raise KeyError('Key %s does not exist.' % (key))
                else:
                    return default
            space.branch_create_empty(key)
            space.checkout_branch(key)
            space.pull(branch=key)
        # self.status()
        self.from_cache_dir_to_path(dst)
        # self.status()
        # input()
        space.checkout_branch(br)
        # self.status()
        # input()
    def from_cache_dir_to_path(self,dst):
        fs=os.listdir(self.cache_dir)
        fs.remove('.git')
        for name in fs:
            path=self.cache_dir+'/'+name
            self._copy(path,dst)
    def _copy_to_cache_dir(self,paths):
        self._copy(paths,self.cache_dir)
    def _copy(self,paths,dst,overwrite=False):
        from wk import copy_dir,copy_file
        if isinstance(paths,str):
            paths=[paths]
        for path in paths:
            if os.path.isdir(path):
                copy_dir(path,dst)
            else:
                copy_file(path,dst,overwrite=overwrite)

def main():
    test()

    pass


def test():
    url=''
    # r=GitSpace.openSpace('MyCloudSpace',url)
    # r=GitSpace.openSpace('piudb',url)
    r = GitSpace.clone(url, overwrite=True)
    # f=r.open('testgit.txt','a')
    # f.write('hi\n')
    # f.close()
    r.pull()
    # r.push()


def demo():
    url="git://github.com/Peiiii/piudb"
    # git.clone(url)
    import os
    rpath = os.path.basename(url)
    open(rpath + '/testgit.txt', 'a').write('hi\n')
    r = Repo(rpath)
    git.add(r, rpath + '/testgit.txt')
    git.commit(r, b'sample commit')
    git.push(r, url, 'master')


if __name__ == '__main__':
    # demo()
    main()



def smart_update_dict(dic1={},dic2={}):
    for k,v in dic2.items():
        if not k in dic1.keys():
            dic1[k]=v
        else:
            if isinstance(dic1[k],dict) and isinstance(dic2[k],dict):
                smart_update_dict(dic1[k],dic2[k])
            else:
                dic1[k]=v
def inrange(n,rg):
    if n>=rg[0] and n<= rg[1]:return True
    return False
def split_dict(dic,keys):
    dic2={}
    for key in keys:
        if key in dic.keys():
            dic2[key]=dic[key]
    return dic2



from flask import Blueprint, Flask, request, send_file, abort, redirect
from wk.web.resources import get_template_by_name, default_static_dir
from wk.web.utils import join_path, rename_func
import uuid, os, logging,inspect,copy


class Application(Flask):
    def __init__(self, import_name=None, enable_CORS=True, host_pkg_resource=True, *args, **kwargs):
        super().__init__(import_name=import_name, *args, **kwargs)
        if enable_CORS:
            try:
                from flask_cors import CORS
                CORS(self, resources=r'/*')
            except:
                logging.warning("CORS is enabled but Flask_cors is not found, install it!")
        self.sitemap = {}
        self.static_map={}
        if host_pkg_resource:
            self.host_pkg_resource()

    def register_blueprint(self, blueprint, url_prefix=None, **options):
        url_prefix = url_prefix or blueprint.url_prefix
        Flask.register_blueprint(self, blueprint, url_prefix=url_prefix, **options)

    def run(self, host="127.0.0.1", port=80, debug=None, load_dotenv=True, **options):
        self.host = host
        self.port = port
        self.debug = debug
        Flask.run(self, host, port, debug, load_dotenv)

    def get_sitemap(self):
        return self.sitemap
    def host_pkg_resource(self):
        self.add_static(url_prefix="/pkg-resource", static_dir=default_static_dir)

    def add_static(self, url_prefix='/files', static_dir='./'):
        self.config_statics({url_prefix: static_dir})

    def config_statics(self, static_map={}):
        self.host_statics(static_map)

    def host_statics(self, static_map={}):
        self.static_map.update(static_map)
        for k, v in static_map.items():
            self._add_static(url_prefix=k, static_dir=v)

    def _add_static(self, url_prefix='/files', static_dir='./', template=None):
        template = get_template_by_name("files") if not template else template
        url_prefix = url_prefix.rstrip('/')

        @self.route(url_prefix + '/', defaults={'req_path': ''})
        @self.route(url_prefix + join_path('/', '<path:req_path>'))
        @rename_func("dir_listing_" + uuid.uuid4().hex)
        def dir_listing(req_path):
            BASE_DIR = static_dir
            abs_path = os.path.join(BASE_DIR, req_path)
            abs_path=os.path.abspath(abs_path)
            if not os.path.exists(abs_path):
                return abort(404)
            if os.path.isfile(abs_path):
                return send_file(abs_path)
            if os.path.isdir(abs_path):
                fns = os.listdir(abs_path)
                fps = [join_path(url_prefix, req_path, f) for f in fns]
                return template.render(files=zip(fps, fns))



class PredfinedKeysMetaClass(type):
    def __new__(cls, name ,bases,attrs):
        key='__predefined_keys__'
        last_base=bases[-1]
        if hasattr(last_base,key):
            base_dic=getattr(last_base,key)
        else:
            base_dic={}
        dic=copy.deepcopy(base_dic)
        for k,v in attrs.items():
            if not inspect.isfunction(v):
                if not k.startswith('__'):
                    dic[k]=v
        attrs[key]=dic
        return type.__new__(cls,name,bases,attrs)
class BpAtrribute:
    def __init__(self,obj=None,inherit=True,run_when_making_class=False):
        self.obj=obj
        if obj is not None:
            inherit=False
        self.inherit=inherit
        self.run_when_making_class=run_when_making_class
class MyBlueprint(Blueprint,metaclass=PredfinedKeysMetaClass):
    import_name = None
    name = None
    add_to_sitemap = False
    url_prefix = None
    host_pkg_resource = True
    static_map = {}
    nickname = None
    enable_CORS = True
    def __init__(self, import_name=None, name=None,add_to_sitemap=None, url_prefix=None, host_pkg_resource=None, static_map={},
                 nickname=None, enable_CORS=None, **kwargs):


        predefined_keys=copy.deepcopy(self.__predefined_keys__)
        # print(predefined_keys)
        arg_dict=dict(import_name=import_name,name=name,add_to_sitemap=add_to_sitemap, url_prefix=url_prefix,
                      host_pkg_resource=host_pkg_resource, static_map=static_map,
                 nickname=nickname, enable_CORS=enable_CORS)
        res_dict={}
        for k,v in arg_dict.items():
            if isinstance(v,dict):
                pre_val=predefined_keys.get(k,None) or {}
                v = v or {}
                v.update(pre_val)
                res_dict[k]=v
            else:
                if v is not None:
                    res_dict[k]=v
                else:
                    res_dict[k]=predefined_keys.get(k,None)
        # print(res_dict)
        import_name=res_dict['import_name']
        name=res_dict['name']
        add_to_sitemap=res_dict['add_to_sitemap']
        url_prefix=res_dict['url_prefix']
        host_pkg_resource=res_dict['host_pkg_resource']
        static_map=res_dict['static_map']
        nickname=res_dict['nickname']
        enable_CORS=res_dict['enable_CORS']

        if not import_name: import_name = "__main__"
        if not name: name = self.__class__.__name__ + uuid.uuid4().hex
        super().__init__(name=name, import_name=import_name, url_prefix=url_prefix, **kwargs)
        self.static_map = {}
        self.nickname = nickname
        self.blueprints={}
        self._blueprint_order=[]
        self.add_to_sitemap=add_to_sitemap
        self.visit_link=None
        self.host_statics(static_map)
        self.enable_CORS = enable_CORS
        if host_pkg_resource:
            self.host_pkg_resource()
        self.app = Application(self.import_name, enable_CORS=self.enable_CORS)
        self.add_handlers()

    def add_handlers(self):
        pass
    def get_visit_link(self):
        pass
    def get_url(self,url=''):
        from wk.basic import standard_path
        return standard_path(self.url_prefix+'/'+url)
    def register(self, app, options, first_registration=False):
        if not hasattr(app, 'sitemap'):
            app.sitemap = {}
        self.app=app
        name = self.nickname if self.nickname else self.name
        if self.add_to_sitemap:
            app.sitemap[name] =self.get_visit_link() or self.visit_link or self.url_prefix
        Blueprint.register(self, app, options, first_registration)

    def run(self, host="127.0.0.1", port=80, debug=False, show_url_map=True):
        self.host = host
        self.port = port
        self.debug = debug
        self.app.register_blueprint(self, url_prefix=self.url_prefix)
        if show_url_map:
            print(self.app.url_map)
        self.app.run(host=host, port=port, debug=self.debug)

    def host_pkg_resource(self):
        self.add_static(url_prefix="/pkg-resource", static_dir=default_static_dir)

    def add_static(self, url_prefix='/files', static_dir='./'):
        self.config_statics({url_prefix: static_dir})

    def config_statics(self, static_map={}):
        self.host_statics(static_map)

    def host_statics(self, static_map={}):
        self.static_map.update(static_map)
        for k, v in static_map.items():
            self._add_static(url_prefix=k, static_dir=v)

    def _add_static(self, url_prefix='/files', static_dir='./', template=None):
        template = get_template_by_name("files") if not template else template
        url_prefix = url_prefix.rstrip('/')
        @self.route(url_prefix + '/', defaults={'req_path': ''})
        @self.route(url_prefix + join_path('/', '<path:req_path>'))
        @rename_func("dir_listing_" + uuid.uuid4().hex)
        def dir_listing(req_path):
            # print("req_path:",req_path)
            BASE_DIR = static_dir
            abs_path = os.path.join(BASE_DIR, req_path)
            abs_path=os.path.abspath(abs_path)
            # print(BASE_DIR,abs_path)
            if not os.path.exists(abs_path):
                return abort(404)
            if os.path.isfile(abs_path):
                return send_file(abs_path)
            if os.path.isdir(abs_path):
                fns = os.listdir(abs_path)
                fps = [join_path(self.url_prefix, url_prefix, req_path, f) for f in fns]
                return template.render(files=zip(fps, fns))


if __name__ == '__main__':
    bp = MyBlueprint(__name__, url_prefix='/', static_map={"/": "../../"})
    app = Application(__name__)
    app.register_blueprint(bp, url_prefix='/files')
    app.run()



from wk.extra.linux import *
if __name__ == '__main__':
    ip = get_local_ip()
    port = 80
    clean_port(port)
    files_dir = r'E:\LearningResources\tmp\高中学习交流群资源'
    from wk.web.applications.FileServer import FileServer
    app = FileServer(__name__, serve_root=files_dir)
    app.run(host=ip,port=port)




def pickle_dump(obj,fp):
    import pickle
    with open(fp,'wb') as f:
        pickle.dump(obj,f)
def pickle_load(fp):
    import pickle
    with open(fp,'rb') as f:
        return pickle.load(f)
def json_load(f,encoding='utf-8',*args,**kwargs):
    import json
    with open(f,'r',encoding=encoding) as fp:
        return json.load(fp,*args,**kwargs)
def json_dump(obj,fp,encoding='utf-8',*args,**kwargs):
    import json
    with open(fp,'w',encoding=encoding) as f:
        json.dump(obj,f,*args,**kwargs)
def read_txt(path,encoding='utf-8'):
    with open(path,'r',encoding=encoding) as f:
        return f.read()
def write_txt(txt,path,encoding='utf-8'):
    with open(path,'w',encoding=encoding) as f:
        f.write(txt)
def write_config(config,path,splitchar='='):
    '''config is a dict,'''
    open(path,'w').close()
    f=open(path,'a')
    for k,v in config.items():
        if isinstance(v,str):
            f.write('%s %s %s\n' % (key,splitchar,value))
        else:
            f.write('[%s]\n'%(k))
            assert isinstance(v,dict)
            for key,value in v.items():
                f.write('%s %s %s\n' % (key,splitchar,value))
def read_config(path,splitchar='=',comment_tag='#'):
    f=open(path,'r')
    config={}
    section_open=False
    current_section=None
    while True:
        s=f.readline()
        if not s:
            break
        if s.strip()=='':
            continue
        s=s.strip()
        if s.startswith('#'):
            continue
        if s.startswith('[') and s.endswith(']'):
            s=s[1:-1]
            assert '[' not in s and ']' not in s
            current_section=s
            config[current_section]={}
            section_open=True
            continue
        key,value=s.split(splitchar)
        key=key.strip()
        value=value.strip()
        if section_open:
            config[current_section][key]=value
        else:
            config[key]=value
    f.close()
    return config
def load_simple_config(fp,line_split='\n',pair_split='=',encoding="utf-8",comment_tags=['#',';']):
    '''
    a=1
    b=2
    '''
    with open(fp,'r',encoding=encoding) as f:
        lines=f.read().strip().split(line_split)
        dic={}
        for line in lines:
            if not line.strip():continue
            is_comment=False
            for tag in comment_tags:
                if line.strip().startswith(tag):
                    is_comment=True
                    break
            if is_comment:continue
            line=line.strip()
            key,value=line.split(pair_split,maxsplit=1)
            key=key.strip()
            value=value.strip()
            dic[key]=value
        return dic
def dump_iterables_to_string(lis,template):
    if not template:
        return lis
    assert isinstance(template, (list, tuple, set))
    length = len(template)
    next_template = None
    cls = template[0]
    if length == 1:
        return str(lis)
    else:
        if cls in [list, set, tuple]:
            lis=list(lis)
            assert length>=2
            splitchar=template[1]
            if length>=3:
                next_template=template[2]
            if next_template:
                for i,item in enumerate(lis):
                    item = dump_iterables_to_string(item,next_template)
                    lis[i]=item
            text = splitchar.join(lis)
            return text
        elif cls in [dict]:
            assert length>=3
            item_split=template[1]
            kv_split=template[2]
            items=[]
            for k,v in lis.items():
                if next_template:
                    v=dump_iterables_to_string(v,next_template)
                text=kv_split.join([k,v])
                items.append(text)
            text=item_split.join(items)
            return text

def load_iterables_from_string(text='',template=[list,'\n']):
    '''
    # [list,'\n',[dict,' ',':',[list,' ',[list,',']]]]
    '''
    if not template:
        return text
    assert isinstance(template, (list, tuple, set))
    length = len(template)
    next_template=None
    cls = template[0]
    if length==1:
        return cls(text)
    else:
        if cls in [list,set,tuple]:
            assert length>=2
            splitchar=template[1]
            if length>=3:
                next_template=template[2]
            items=text.split(splitchar)
            if next_template:
                for i,item in enumerate(items):
                    items[i]=load_iterables_from_string(item,next_template)
            items=cls(items)
            return items
        elif cls in [dict]:
            assert length >= 3
            item_split=template[1]
            kv_split=template[2]
            if length>=4:
                next_template=template[3]
            dic={}
            items=text.split(item_split)
            for item in items:
                k,v =item.split(kv_split,maxsplit=1)
                if next_template:
                    v = load_iterables_from_string(v, next_template)
                dic[k]=v
            return dic
def concat_text_files(src_files,dst_file,split_char=''):
    texts=[]
    for src in src_files:
        txt=read_txt(src)
        texts.append(txt)
    with open(dst_file,'w',encoding='utf-8') as f:
        f.write(split_char.join(texts))




from .StoreItem import *
from .utils import generate_hash



_BRANCH_DICT='branch_dict'
_FAKE2TRUE='fake2true'
_TRUE2FAKE='true2fake'
USER_HOME=os.path.expanduser('~')
STORE_HOME=USER_HOME+'/.store'
BRANCH_DICTS_DIR=STORE_HOME+'/BranchDicts'

def get_default_path(remote_location,remote_branch):
    name = generate_hash(remote_location + '/' + remote_branch)
    path = BRANCH_DICTS_DIR + '/' + name
    return path


class BranchDict(StoreFolder):
    def __init__(self,path=None,remote_location=None,remote_branch=None):
        # path=USER_HOME+'/.store'
        remote_branch=remote_branch or _BRANCH_DICT
        if not path:
            path=get_default_path(remote_location,remote_branch)
        super().__init__(path,remote_location,remote_branch)
        self._init_branch_dict()
    def _init_branch_dict(self):
        '''
        Make sure we have local branch dict same with remote one.
        '''
        # print("files:",self.listdir())
        # print("dict:",self._read_fake2true_dict())
        if not self.rbl.branch_exists(self.remote_branch):
            self.openFiledict(_FAKE2TRUE)
            self.openFiledict(_TRUE2FAKE)

            self._push_self()
            self.rbl.branch_add(self.remote_branch)
        else:
            self._try_pull_remote()
            if not self.exists(_FAKE2TRUE) or not self.exists(_TRUE2FAKE):
                self.openFiledict(_FAKE2TRUE)
                self.openFiledict(_TRUE2FAKE)
                self._push_self()
    def _sync_dict(self):
        return self._try_pull_remote()
    def _concat_fakes(self,parent,fake):
        return parent+StoreFolder.delimiter+fake
    def _generate_hash(self,fake):
        import hashlib
        m=hashlib.md5()
        def gen():
            m.update(fake.encode('utf-8'))
            return m.hexdigest()[:10]
        while True:
            true=gen()
            if not true in self.trues():
                return true

    def _update_branch_dict(self,fake,true):
        self._try_pull_remote()
        fake2true=self._read_fake2true_dict()
        true2fake=self._read_true2fake_dict()
        fake2true[fake]=true
        true2fake[true]=fake
        self._push_self()
    def true(self,fake):
        if fake in self.fakes():
            return self.fake2true(fake)
        else:
            true=self._generate_hash(fake)
            self.set(fake,true)
            return true
    def _read_fake2true_dict(self):
        return self.openFiledict(_FAKE2TRUE)
    def _read_true2fake_dict(self):
        return self.openFiledict(_TRUE2FAKE)
    def fake2true(self,fake):
        return self._read_fake2true_dict()[fake]
    def true2fake(self,true):
        return self._read_true2fake_dict()[true]
    def trues(self):
        return self._read_true2fake_dict().keys()
    def fakes(self):
        return self._read_fake2true_dict().keys()
    def set(self,fake,true):
        return self._update_branch_dict(fake,true)

class BranchDict_old(StoreFolder):
    def __init__(self,path,remote_location=None,remote_branch=None):
        remote_branch=remote_branch or _BRANCH_DICT
        super().__init__(path,remote_location,remote_branch)
        self._init_branch_dict()
    def _init_branch_dict(self):
        '''
        Make sure we have local branch dict same with remote one.
        '''
        self._try_pull_remote()
        # print("bd:", self._read_fake2true_dict())
        self._read_remote_branch_list(pull=True)
        self._pull_else_push_self()
        # self._try_pull_remote()
        # print("bd:", self._read_fake2true_dict())
        if self.is_empty():
            self.openFiledict(_FAKE2TRUE)
            self.openFiledict(_TRUE2FAKE)
            self._push_self()
        # print("bd:", self._read_fake2true_dict())
    def _sync_dict(self):
        return self._try_pull_remote()
    def _concat_fakes(self,parent,fake):
        return parent+StoreFolder.delimiter+fake
    def _generate_hash(self,fake):
        import hashlib
        m=hashlib.md5()
        def gen():
            m.update(fake.encode('utf-8'))
            return m.hexdigest()[:10]
        while True:
            true=gen()
            if not true in self.trues():
                return true

    def _update_branch_dict(self,fake,true):
        self._try_pull_remote()
        fake2true=self._read_fake2true_dict()
        true2fake=self._read_true2fake_dict()
        fake2true[fake]=true
        true2fake[true]=fake
        self._push_self()
    def true(self,fake):
        if fake in self.fakes():
            return self.fake2true(fake)
        else:
            true=self._generate_hash(fake)
            self.set(fake,true)
            return true
    def _read_fake2true_dict(self):
        return self.openFiledict(_FAKE2TRUE)
    def _read_true2fake_dict(self):
        return self.openFiledict(_TRUE2FAKE)
    def fake2true(self,fake):
        return self._read_fake2true_dict()[fake]
    def true2fake(self,true):
        return self._read_true2fake_dict()[true]
    def trues(self):
        return self._read_true2fake_dict().keys()
    def fakes(self):
        return self._read_fake2true_dict().keys()
    def set(self,fake,true):
        return self._update_branch_dict(fake,true)




import os,shutil,glob
import inspect
import time

def is_empty_dir(path):
    assert os.path.exists(path)
    if not os.path.isdir(path):
        return False
    if len(os.listdir(path)):
        return False
    else:
        return True
def _copy_dir(src,dst):
    assert os.path.exists(src)
    assert os.path.exists(dst)
    for name in os.listdir(src):
        path=os.path.join(src,name)
        new_path=os.path.join(dst,name)
        if os.path.isdir(path):
            os.mkdir(new_path)
            _copy_dir(path,new_path)
        elif os.path.isfile(path):
            shutil.copy(path,new_path)
def copy_file(src,dst,overwrite=False):
    '''
    if dst is dir: copy into dir
    if dst is file and overwrite: overwrite
    if dst doesn't exist : parent should exist, use shutil.copy
    '''
    assert os.path.isfile(src)
    if os.path.exists(dst):
        if os.path.isdir(dst):
            shutil.copy(src,dst+'/'+os.path.basename(src))
        else:
            if overwrite:
                shutil.copy(src,dst)
            else:
                raise FileExistsError('%s is an existed file.'%(dst))
    else:
        parent=os.path.dirname(dst)
        if parent=='':
            parent='./'
        assert os.path.exists(parent)
        shutil.copy(src,dst)


def copy_dir(src,dst):
    '''
    if dst exists, copy src into dst,
    else mkdir dst and copy src's contents into dst
    '''
    assert os.path.exists(src)
    src=os.path.abspath(src)
    name=os.path.basename(src)
    if os.path.exists(dst):
        assert os.path.isdir(dst)
        newdir=os.path.join(dst,name)
        os.mkdir(newdir)
        _copy_dir(src,newdir)
    else:
        parent_dir=os.path.dirname(dst)
        # assert os.path.exists(parent_dir)
        os.mkdir(dst)
        _copy_dir(src,dst)

def copy_fsitem(src,dst,overwrite=False):
    assert os.path.exists(src)
    if os.path.isdir(src):
        copy_dir(src,dst)
    else:
        copy_file(src,dst,overwrite=overwrite)
def remove_fsitem(path):
    if os.path.isdir(path):
        return shutil.rmtree(path)
    else:
        time=0
        while True:
            try:
                return os.remove(path)
            except:
                time+=1
                if time==3:
                    raise
def merge_dirs(src_dirs,dst_dir):
    if  os.path.exists(dst_dir):
        shutil.rmtree(dst_dir)
        a=0
    os.makedirs(dst_dir)
    for dir in src_dirs:
        fs=glob.glob(dir+'/*')
        copy_files_to(fs,dst_dir)

def batch_rename_files(src_dir,handler,out_dir=None,glob_str='*',sort_key_func=None):
    src_dir=os.path.abspath(src_dir)
    fs=glob.glob(src_dir+'/'+glob_str)
    if sort_key_func:
        fs.sort(key=sort_key_func)
    if not out_dir:
        out_dir=os.path.dirname(src_dir)+'/'+os.path.basename(src_dir)+'_rename_output'
        remake(out_dir)
    args=inspect.getfullargspec(handler)[0]
    for i,f in enumerate(fs):
        name=os.path.basename(f)
        if len(args)==1:
            name2=handler(name)
        else:
            name2=handler(name,i)
        if name2:
            f2=out_dir+'/'+name2
            shutil.copy(f,f2)
            print(i,f,f2)
    print('finished.')



def copy_files_to(files,dst,overwrite=False):
    if not os.path.exists(dst):
        os.makedirs(dst)
    for i,f in enumerate(files):
        fn=os.path.basename(f)
        f2=dst+'/'+fn
        if os.path.exists(f2):
            if os.path.samefile(f, f2):
                print("ignoring same file:", f, f2)
                continue
            if not overwrite:
                raise Exception("file %s already exists."%(f2))
            else:
                print("overwriting %s to %s ..."%(f,f2))
                os.remove(f2)
        shutil.copy(f,f2)
def tranverse_files(dir,func):
    fs=glob.glob(dir+'/*')
    for f in fs:
        if os.path.isfile(f):
            func(f)
        elif os.path.isdir(f):
            tranverse_files(f,func)
def tranverse_dirs(dir,func):
    fs=glob.glob(dir+'/*')
    for f in fs:
        if os.path.isfile(f):
            pass
        elif os.path.isdir(f):
            func(f)
            tranverse_dirs(f,func)
def remove(path,not_exist_ok=True):
    if os.path.exists(path):
        if os.path.isdir(path) :
            shutil.rmtree(path)
        else:
            os.remove(path)
    elif not not_exist_ok:
        raise Exception('File or dir %s does not exist'%(path))
def remake(dir,speep_time=0):
    if os.path.exists(dir):
        shutil.rmtree(dir)
        time.sleep(speep_time)
    os.makedirs(dir)
def compare_dirs(src1,src2,dst='./compare_results'):
    files1=glob.glob(src1+'/*.*')
    files2=glob.glob(src2+'/*.*')
    same1=dst+'/1_same'
    same2=dst+'/2_same'
    diff1=dst+'/1_diff'
    diff2=dst+'/2_diff'
    remake(same1)
    remake(same2)
    remake(diff1)
    remake(diff2)
    names1=[os.path.basename(f) for f in files1]
    names2=[os.path.basename(f) for f in files2]
    ds1=set(names1)-set(names2)
    ds2=set(names2)-set(names1)
    ss=set(names1).intersection(set(names2))
    dl1=[src1+'/'+f for f in ds1]
    dl2=[src2+'/'+f for f in ds2]
    sl1=[src1+'/'+f for f in ss]
    sl2=[src2+'/'+f for f in ss]
    copy_files_to(dl1,diff1)
    copy_files_to(dl2,diff2)
    copy_files_to(sl1,same1)
    copy_files_to(sl2,same2)



# -*- coding=UTF-8 -*-


def heart(word):

    print('\n'.join([''.join([(word[(x - y) % len(word)] if ((x * 0.05) ** 2 + (y * 0.1) ** 2 - 1) ** 3 - (
                x * 0.05) ** 2 * (y * 0.1) ** 3 <= 0 else ' ') for x in range(-30, 30)]) for y in range(30, -30, -1)]))
def heart2(title='I Love You'):
    import turtle
    import time
    def liujia():
        for i in range(200):
            turtle.right(1)
            turtle.forward(1)

    turtle.color('red', 'pink')
    turtle.pensize(2)
    turtle.speed(10)
    turtle.goto(0, 0)

    turtle.begin_fill()
    turtle.left(140)
    turtle.forward(112)
    liujia()
    turtle.left(120)
    liujia()
    turtle.forward(112)
    turtle.end_fill()
    turtle.pensize(5)
    turtle.up()
    turtle.goto(-50, 142.7)
    turtle.left(50)
    turtle.down()
    turtle.forward(60)
    turtle.left(90)
    turtle.forward(25)
    turtle.up()
    turtle.goto(37.5, 142.7)
    turtle.down()
    turtle.forward(25)
    turtle.up()
    turtle.goto(50, 142.7)
    turtle.right(90)
    turtle.down()
    turtle.forward(60)
    for i in range(20):
        turtle.right(7.8)
        turtle.forward(0.3)
    turtle.forward(8)
    turtle.up()
    turtle.goto(100, -10)
    turtle.write(title)
    turtle.exitonclick()

def heart3():
    import turtle
    import time

    def draw_circle():
        for i in range(400):
            turtle.right(0.5)
            turtle.forward(1)

    def draw_love():
        #    turtle.color('red','darkred')
        #    turtle.pensize(1)
        turtle.pen(fillcolor="black", pencolor="red", pensize=8)
        turtle.speed(2000)
        turtle.goto(0, 0)
        turtle.begin_fill()
        turtle.left(140)
        turtle.forward(224)
        draw_circle()
        turtle.left(120)
        draw_circle()
        turtle.forward(224)
        turtle.end_fill()
        turtle.write("I Love you")
        time.sleep(2)
        turtle.up()
        turtle.goto(150, 20)
        turtle.color('black')
        turtle.write('相夕妄想 来日方长 很喜欢你 到此为止', font=("微软雅黑", 18, "normal"))
        time.sleep(2)

    def draw_abc():
        turtle.fillcolor("black")
        turtle.pencolor("red")
        turtle.pensize(10)
        turtle.speed(1)
        turtle.up()
        turtle.goto(0, -50)
        turtle.down()
        turtle.begin_fill()
        turtle.circle(45)
        turtle.end_fill()
        time.sleep(2)

    def word():
        turtle.up()
        turtle.goto(-100, 200)
        turtle.color("red")
        turtle.pensize(4)
        #   turtle.down()
        turtle.write('宝贝，你以为你是唯一吗？', font=("隶书", 18, "bold"))
        time.sleep(10)

    draw_love()
    draw_abc()
    word()
def heart4():
    import turtle
    import time

    # 画心形圆弧
    def hart_arc():
        for i in range(200):
            turtle.right(1)
            turtle.forward(2)

    def move_pen_position(x, y):
        turtle.hideturtle()  # 隐藏画笔（先）
        turtle.up()  # 提笔
        turtle.goto(x, y)  # 移动画笔到指定起始坐标（窗口中心为0,0）
        turtle.down()  # 下笔
        turtle.showturtle()  # 显示画笔

    love = input("请输入表白话语，默认为‘I Love You’：")
    signature = input("请签署你的大名，不填写默认不显示：")

    if love == '':
        love = 'I Love You'

    # 初始化
    turtle.setup(width=800, height=500)  # 窗口（画布）大小
    turtle.color('red', 'pink')  # 画笔颜色
    turtle.pensize(3)  # 画笔粗细
    turtle.speed(1)  # 描绘速度
    # 初始化画笔起始坐标
    move_pen_position(x=0, y=-180)  # 移动画笔位置
    turtle.left(140)  # 向左旋转140度

    turtle.begin_fill()  # 标记背景填充位置

    # 画心形直线（ 左下方 ）
    turtle.forward(224)  # 向前移动画笔，长度为224
    # 画爱心圆弧
    hart_arc()  # 左侧圆弧
    turtle.left(120)  # 调整画笔角度
    hart_arc()  # 右侧圆弧
    # 画心形直线（ 右下方 ）
    turtle.forward(224)

    turtle.end_fill()  # 标记背景填充结束位置

    # 在心形中写上表白话语
    move_pen_position(0, 0)  # 表白语位置
    turtle.hideturtle()  # 隐藏画笔
    turtle.color('#CD5C5C', 'pink')  # 字体颜色
    # font:设定字体、尺寸（电脑下存在的字体都可设置）  align:中心对齐
    turtle.write(love, font=('Arial', 30, 'bold'), align="center")

    # 签写署名
    if signature != '':
        turtle.color('red', 'pink')
        time.sleep(2)
        move_pen_position(180, -180)
        turtle.hideturtle()  # 隐藏画笔
        turtle.write(signature, font=('Arial', 20), align="center")

    # 点击窗口关闭程序
    window = turtle.Screen()
    window.exitonclick()
def tree1():
    import turtle as tl
    def draw_smalltree(tree_length, tree_angle):
        '''
        绘制分形树函数
        '''
        if tree_length >= 3:
            tl.forward(tree_length)  # 往前画
            tl.right(tree_angle)  # 往右转
            draw_smalltree(tree_length - 10, tree_angle)  # 画下一枝，直到画到树枝长小于3

            tl.left(2 * tree_angle)  # 转向画左
            draw_smalltree(tree_length - 10, tree_angle)  # 直到画到树枝长小于3

            tl.rt(tree_angle)  # 转到正向上的方向，然后回溯到上一层
            if tree_length <= 30:  # 树枝长小于30，可以当作树叶了，树叶部分为绿色
                tl.pencolor('green')
            if tree_length > 30:
                tl.pencolor('brown')  # 树干部分为棕色
            tl.backward(tree_length)  # 往回画，回溯到上一层

    tl.penup()
    # tl.pencolor('green')
    tl.left(90)  # 因为树是往上的，所以先把方向转左
    tl.backward(250)  # 把起点放到底部
    tl.pendown()
    tree_length = 100  # 我设置的最长树干为100
    tree_angle = 20  # 树枝分叉角度，我设为20
    draw_smalltree(tree_length, tree_angle)
    tl.exitonclick()  # 点击才关闭画画窗口

def tree2():
    import turtle

    def draw_brach(brach_length):

        if brach_length > 5:
            if brach_length < 40:
                turtle.color('green')

            else:
                turtle.color('red')

            # 绘制右侧的树枝
            turtle.forward(brach_length)
            print('向前', brach_length)
            turtle.right(25)
            print('右转20')
            draw_brach(brach_length - 15)
            # 绘制左侧的树枝
            turtle.left(50)
            print('左转40')
            draw_brach(brach_length - 15)

            if brach_length < 40:
                turtle.color('green')

            else:
                turtle.color('red')

            # 返回之前的树枝上
            turtle.right(25)
            print('右转20')
            turtle.backward(brach_length)
            print('返回', brach_length)

    def main():
        turtle.left(90)
        turtle.penup()
        turtle.backward(150)
        turtle.pendown()
        turtle.color('red')

        draw_brach(100)

        turtle.exitonclick()
    main()

def tree3():
    import turtle as tl

    def draw_smalltree(tree_length, tree_angle, tree_wide):
        '''
        绘制分形树函数
        '''
        if tree_length >= 5:
            tl.pensize(tree_wide)
            tl.forward(tree_length)  # 往前画
            tl.right(tree_angle)  # 往右转
            draw_smalltree(tree_length - 10, tree_angle, tree_wide * 2 / 3)  # 画下一枝，直到画到树枝长小于3

            tl.left(2 * tree_angle)  # 转向画左
            draw_smalltree(tree_length - 10, tree_angle, tree_wide * 2 / 3)  # 直到画到树枝长小于3

            tl.rt(tree_angle)  # 转到正向上的方向，然后回溯到上一层
            if tree_length <= 30:  # 树枝长小于30，可以当作树叶了，树叶部分为绿色
                tl.pencolor('green')
            if tree_length > 30:
                tl.pencolor('brown')  # 树干部分为棕色

            tl.pensize(tree_wide)
            tl.backward(tree_length)  # 往回画，回溯到上一层

    def main():
        tl.screensize(100, 100, "black")  # 画布大小
        tl.penup()
        tl.left(90)  # 因为树是往上的，所以先把方向转左
        tl.backward(250)  # 把起点放到底部
        tl.pendown()
        tl.pencolor('brown')
        tl.speed(10)
        tree_length = 80  # 我设置的最长树干为80
        tree_angle = 20  # 树枝分叉角度，我设为20
        tree_wide = 5  # 树枝粗度
        draw_smalltree(tree_length, tree_angle, tree_wide)
        tl.exitonclick()  # 点击才关闭画画窗口
    main()

def tree4():
    '''表白用'''

    import turtle
    import random
    def love(x, y):  # 在(x,y)处画爱心lalala
        lv = turtle.Turtle()
        lv.hideturtle()
        lv.up()
        lv.goto(x, y)  # 定位到(x,y)

        def curvemove():  # 画圆弧
            for i in range(20):
                lv.right(10)
                lv.forward(2)

        lv.color('red', 'pink')
        lv.speed(10000000)
        lv.pensize(1)
        # 开始画爱心lalala
        lv.down()
        lv.begin_fill()
        lv.left(140)
        lv.forward(22)
        curvemove()
        lv.left(120)
        curvemove()
        lv.forward(22)
        lv.write("GPA", font=("Arial", 12, "normal"), align="center")  # 写上表白的人的名字
        lv.left(140)  # 画完复位
        lv.end_fill()

    def tree(branchLen, t):
        if branchLen > 5:  # 剩余树枝太少要结束递归
            if branchLen < 20:  # 如果树枝剩余长度较短则变绿
                t.color("green")
                t.pensize(random.uniform((branchLen + 5) / 4 - 2, (branchLen + 6) / 4 + 5))
                t.down()
                t.forward(branchLen)
                love(t.xcor(), t.ycor())  # 传输现在turtle的坐标
                t.up()
                t.backward(branchLen)
                t.color("brown")
                return
            t.pensize(random.uniform((branchLen + 5) / 4 - 2, (branchLen + 6) / 4 + 5))
            t.down()
            t.forward(branchLen)
            # 以下递归
            ang = random.uniform(15, 45)
            t.right(ang)
            tree(branchLen - random.uniform(12, 16), t)  # 随机决定减小长度
            t.left(2 * ang)
            tree(branchLen - random.uniform(12, 16), t)  # 随机决定减小长度
            t.right(ang)
            t.up()
            t.backward(branchLen)

    myWin = turtle.Screen()
    t = turtle.Turtle()
    t.hideturtle()
    t.speed(1000)
    t.left(90)
    t.up()
    t.backward(200)
    t.down()
    t.color("brown")
    t.pensize(32)
    t.forward(60)
    tree(100, t)
    myWin.exitonclick()

def rose1(love=None):
    import turtle
    from turtle import pensize,showturtle,penup,pendown,left,fd,right,fillcolor,begin_fill,circle,setheading,end_fill,done

    def move_pen_position(x, y):
        turtle.hideturtle()  # 隐藏画笔（先）
        turtle.up()  # 提笔
        turtle.goto(x, y)  # 移动画笔到指定起始坐标（窗口中心为0,0）
        turtle.down()  # 下笔
        turtle.showturtle()  # 显示画笔
    # 初始化一下，摆好姿势...
    pensize(2)
    showturtle()
    penup()
    left(90)
    fd(200)
    pendown()
    right(90)
    # 来一个火辣辣的花蕊，代表我对你深深的爱...
    fillcolor("red")
    begin_fill()
    circle(10, 180)
    circle(25, 110)
    left(50)
    circle(60, 45)
    circle(20, 170)
    right(24)
    fd(30)
    left(10)
    circle(30, 110)
    fd(20)
    left(40)
    circle(90, 70)
    circle(30, 150)
    right(30)
    fd(15)
    circle(80, 90)
    left(15)
    fd(45)
    right(165)
    fd(20)
    left(155)
    circle(150, 80)
    left(50)
    circle(150, 90)
    end_fill()
    # 来一些花瓣，让她爱上我
    left(150)
    circle(-90, 70)
    left(20)
    circle(75, 105)
    setheading(60)
    circle(80, 98)
    circle(-90, 40)
    # 再来一些花瓣，让她爱上我
    left(180)
    circle(90, 40)
    circle(-80, 98)
    setheading(-83)
    # 来一片绿油油的叶子...
    fd(30)
    left(90)
    fd(25)
    left(45)
    fillcolor("green")
    begin_fill()
    circle(-80, 90)
    right(90)
    circle(-80, 90)
    end_fill()
    right(135)
    fd(60)
    left(180)
    fd(85)
    left(90)
    fd(80)
    # 再来一片绿油油的叶子...
    right(90)
    right(45)
    fillcolor("green")
    begin_fill()
    circle(80, 90)
    left(90)
    circle(80, 90)
    end_fill()
    left(135)
    fd(60)
    left(180)
    fd(60)
    left(90)
    circle(200, -60)
    if love:
        move_pen_position(0, 0)  # 表白语位置
        turtle.hideturtle()  # 隐藏画笔
        turtle.color('#CD5C5C', 'pink')  # 字体颜色
        # font:设定字体、尺寸（电脑下存在的字体都可设置）  align:中心对齐
        turtle.write(love, font=('Arial', 30, 'bold'), align="center")
    done()
def peppa():
    import turtle as t
    t.pensize(4)
    t.hideturtle()
    t.colormode(255)
    t.color((255, 155, 192), "pink")
    t.setup(840, 500)
    t.speed(10)

    # 鼻子
    t.pu()
    t.goto(-100, 100)
    t.pd()
    t.seth(-30)
    t.begin_fill()
    a = 0.4
    for i in range(120):
        if 0 <= i < 30 or 60 <= i < 90:
            a = a + 0.08
            t.lt(3)  # 向左转3度
            t.fd(a)  # 向前走a的步长
        else:
            a = a - 0.08
            t.lt(3)
            t.fd(a)
    t.end_fill()

    t.pu()
    t.seth(90)
    t.fd(25)
    t.seth(0)
    t.fd(10)
    t.pd()
    t.pencolor(255, 155, 192)
    t.seth(10)
    t.begin_fill()
    t.circle(5)
    t.color(160, 82, 45)
    t.end_fill()

    t.pu()
    t.seth(0)
    t.fd(20)
    t.pd()
    t.pencolor(255, 155, 192)
    t.seth(10)
    t.begin_fill()
    t.circle(5)
    t.color(160, 82, 45)
    t.end_fill()

    # 头
    t.color((255, 155, 192), "pink")
    t.pu()
    t.seth(90)
    t.fd(41)
    t.seth(0)
    t.fd(0)
    t.pd()
    t.begin_fill()
    t.seth(180)
    t.circle(300, -30)
    t.circle(100, -60)
    t.circle(80, -100)
    t.circle(150, -20)
    t.circle(60, -95)
    t.seth(161)
    t.circle(-300, 15)
    t.pu()
    t.goto(-100, 100)
    t.pd()
    t.seth(-30)
    a = 0.4
    for i in range(60):
        if 0 <= i < 30 or 60 <= i < 90:
            a = a + 0.08
            t.lt(3)  # 向左转3度
            t.fd(a)  # 向前走a的步长
        else:
            a = a - 0.08
            t.lt(3)
            t.fd(a)
    t.end_fill()

    # 耳朵
    t.color((255, 155, 192), "pink")
    t.pu()
    t.seth(90)
    t.fd(-7)
    t.seth(0)
    t.fd(70)
    t.pd()
    t.begin_fill()
    t.seth(100)
    t.circle(-50, 50)
    t.circle(-10, 120)
    t.circle(-50, 54)
    t.end_fill()

    t.pu()
    t.seth(90)
    t.fd(-12)
    t.seth(0)
    t.fd(30)
    t.pd()
    t.begin_fill()
    t.seth(100)
    t.circle(-50, 50)
    t.circle(-10, 120)
    t.circle(-50, 56)
    t.end_fill()

    # 眼睛
    t.color((255, 155, 192), "white")
    t.pu()
    t.seth(90)
    t.fd(-20)
    t.seth(0)
    t.fd(-95)
    t.pd()
    t.begin_fill()
    t.circle(15)
    t.end_fill()

    t.color("black")
    t.pu()
    t.seth(90)
    t.fd(12)
    t.seth(0)
    t.fd(-3)
    t.pd()
    t.begin_fill()
    t.circle(3)
    t.end_fill()

    t.color((255, 155, 192), "white")
    t.pu()
    t.seth(90)
    t.fd(-25)
    t.seth(0)
    t.fd(40)
    t.pd()
    t.begin_fill()
    t.circle(15)
    t.end_fill()

    t.color("black")
    t.pu()
    t.seth(90)
    t.fd(12)
    t.seth(0)
    t.fd(-3)
    t.pd()
    t.begin_fill()
    t.circle(3)
    t.end_fill()

    # 腮
    t.color((255, 155, 192))
    t.pu()
    t.seth(90)
    t.fd(-95)
    t.seth(0)
    t.fd(65)
    t.pd()
    t.begin_fill()
    t.circle(30)
    t.end_fill()

    # 嘴
    t.color(239, 69, 19)
    t.pu()
    t.seth(90)
    t.fd(15)
    t.seth(0)
    t.fd(-100)
    t.pd()
    t.seth(-80)
    t.circle(30, 40)
    t.circle(40, 80)

    # 身体
    t.color("red", (255, 99, 71))
    t.pu()
    t.seth(90)
    t.fd(-20)
    t.seth(0)
    t.fd(-78)
    t.pd()
    t.begin_fill()
    t.seth(-130)
    t.circle(100, 10)
    t.circle(300, 30)
    t.seth(0)
    t.fd(230)
    t.seth(90)
    t.circle(300, 30)
    t.circle(100, 3)
    t.color((255, 155, 192), (255, 100, 100))
    t.seth(-135)
    t.circle(-80, 63)
    t.circle(-150, 24)
    t.end_fill()

    # 手
    t.color((255, 155, 192))
    t.pu()
    t.seth(90)
    t.fd(-40)
    t.seth(0)
    t.fd(-27)
    t.pd()
    t.seth(-160)
    t.circle(300, 15)
    t.pu()
    t.seth(90)
    t.fd(15)
    t.seth(0)
    t.fd(0)
    t.pd()
    t.seth(-10)
    t.circle(-20, 90)

    t.pu()
    t.seth(90)
    t.fd(30)
    t.seth(0)
    t.fd(237)
    t.pd()
    t.seth(-20)
    t.circle(-300, 15)
    t.pu()
    t.seth(90)
    t.fd(20)
    t.seth(0)
    t.fd(0)
    t.pd()
    t.seth(-170)
    t.circle(20, 90)

    # 脚
    t.pensize(10)
    t.color((240, 128, 128))
    t.pu()
    t.seth(90)
    t.fd(-75)
    t.seth(0)
    t.fd(-180)
    t.pd()
    t.seth(-90)
    t.fd(40)
    t.seth(-180)
    t.color("black")
    t.pensize(15)
    t.fd(20)

    t.pensize(10)
    t.color((240, 128, 128))
    t.pu()
    t.seth(90)
    t.fd(40)
    t.seth(0)
    t.fd(90)
    t.pd()
    t.seth(-90)
    t.fd(40)
    t.seth(-180)
    t.color("black")
    t.pensize(15)
    t.fd(20)

    # 尾巴
    t.pensize(4)
    t.color((255, 155, 192))
    t.pu()
    t.seth(90)
    t.fd(70)
    t.seth(0)
    t.fd(95)
    t.pd()
    t.seth(0)
    t.circle(70, 20)
    t.circle(10, 330)
    t.circle(70, 30)

if __name__ == '__main__':
    import fire
    fire.Fire()


import pkg_resources, os
from wk.io import load_simple_config
from .utils import SecureDirPath, PointDict, Path, DirPath
from wk import pkg_info
from jinja2 import Environment, PackageLoader, \
    BaseLoader, FileSystemLoader, \
    ChoiceLoader, DictLoader, PrefixLoader

data_path = DirPath(pkg_info.pkg_data_dir)
pkg_data_path = data_path
pkg_static_path = data_path + '/static'
pkg_document_path = pkg_data_path / 'documents'
pkg_templates_dir = DirPath(data_path) / 'templates'
pkg_js_dir = DirPath(data_path) / 'static' / 'js'
default_templates = PointDict.from_dict({
    'welcome': pkg_resources.resource_filename('wk', 'data/templates/welcome.html'),
    'files': pkg_resources.resource_filename('wk', 'data/templates/files.html'),
    'board': pkg_resources.resource_filename('wk', 'data/templates/board.html'),
    'sitemap': pkg_resources.resource_filename('wk', 'data/templates/sitemap.html'),
    'post': pkg_resources.resource_filename('wk', 'data/templates/post.html'),
    'outsite': pkg_resources.resource_filename('wk', 'data/templates/outsite.html')
})
default_static_dir = SecureDirPath(pkg_info.pkg_dir) / 'data' / 'static'


class TemplateLoader:
    def __init__(self, root):
        self.root = root
        self.env=Environment(loader=FileSystemLoader(self.root))
        self.frame_env=Environment(block_start_string='<%',block_end_string='%>',variable_start_string='<@',variable_end_string='@>',loader=FileSystemLoader(self.root))
    def load_frame(self,fn):
        return self.frame_env.get_template(fn).render()
    def load(self, fn):
        return self.env.get_template(fn)
        # return self.frame_env.get_template(fn).render()
    def load_plain(self,fn):
        path = self.root + '/' + fn
        with open(path, 'r', encoding='utf-8') as f:
            return f.read()
    def subdir(self, name):
        root = self.root + '/' + name
        return TemplateLoader(root)


Parts = TemplateLoader(root=pkg_data_path + "/parts")
Sites = TemplateLoader(root=pkg_static_path + '/sites')


def get_default_template_string(tem):
    return open(default_templates[tem], 'r', encoding='utf-8').read()


env = Environment(loader=PackageLoader('wk.data', 'templates'))
sys_env = env


def get_template_by_name(fn='base'):
    if not '.' in fn: fn += '.html'
    return env.get_template(fn)


def get_template_string_by_name(fn):
    if not fn.endswith('.html'): fn += '.html'
    return (pkg_templates_dir / fn)()


def get_js_string_by_name(fn):
    if not fn.endswith('.js'): fn += '.js'
    return (pkg_js_dir / fn)()


def get_env(path=None):
    path = path or './'
    _loader = ChoiceLoader([
        FileSystemLoader(path),
        PrefixLoader({
            'sys': ChoiceLoader([
                FileSystemLoader(data_path + '/static'),
                FileSystemLoader(data_path + '/templates')
            ]),
            'user': ChoiceLoader([
                FileSystemLoader('data/templates'),
                FileSystemLoader('data/user/templates'),
            ])
        }),
        ChoiceLoader([
            FileSystemLoader('data/templates'),
            FileSystemLoader('data/user/templates'),
            FileSystemLoader(data_path + '/static'),
            FileSystemLoader(data_path + '/templates'),
        ])
    ])
    env = Environment(
        loader=_loader
    )
    return env


pkg_tem_loader = ChoiceLoader([
    FileSystemLoader(data_path + '/static'),
    FileSystemLoader(data_path + '/templates')
])
pkg_env = Environment(
    loader=pkg_tem_loader
)


# class Pages:
#     base=pkg_env.get_template('base.html')
#     links=get_template_by_name('links')
#     view_text_file=pkg_env.get_template('view_file.tem')
#     view_markdown_file=pkg_env.get_template('view_md.tem')
def get_one_exist_path(paths):
    for path in paths:
        if os.path.exists(path):
            return path
    return None


def get_exist_template_from_env(env, tems):
    for tem in tems:
        try:
            return env.get_template(tem)
        except:
            pass


def get_page_template(path):
    if os.path.isdir(path):
        dirname = path
        env = get_env(dirname)
        tem = get_exist_template_from_env(env, ['index.page', 'map.tem'])
        return tem
    else:
        dirname = os.path.dirname(path)
        basename = os.path.basename(path)
        env = get_env(dirname)
        if path.endswith('.page'):
            return env.get_template(basename)


def get_book(path):
    config = load_simple_config(path)
    env = get_env(os.path.dirname(path))
    tem = env.get_template('book.tem')
    os_url = config.get('os_url', '/os')
    book_path = config.get('book_path', './')
    return tem.render(os_url=os_url, book_path=book_path)



import os, shutil, glob, random, json, math, uuid, time, datetime, inspect, re
import jieba
import jieba.analyse
try:
    from textrank4zh import TextRank4Sentence
except:
    TextRank4Sentence=None
from wk.io import db, Piu
from wk import Folder, join_path, PointDict, web, DefaultDict, get_file_info
import wk
from .utils import get_keywords


class User(DefaultDict):
    default = dict(
        id=lambda: uuid.uuid4().hex,
        username=None,
        email=None,
        password=None,
        signature=None,
        avatar=lambda this: 'http://www.gravatar.com/avatar/%s?s=256&d=retro' % (wk.generate_hash(this['email'])),
        followers=0,
        following=0,
        zan=0,
        cai=0,
        favList=[],
        articles=[],
        public_articles=[],
        private_articles=[],
        recycled_articles=[],
        drafts=[],
        link=lambda this: join_path('/u', this['id']),
    )

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

    def publish_article(self, doc_id):
        self['public_articles'].append(doc_id)
        self['private_articles'].remove(doc_id)

    def new_article(self, doc_id):
        self['private_articles'].append(doc_id)

    def delete_recycled_article(self, doc_id):
        self['recycled_articles'].remove(doc_id)

    def delete_private_article(self, doc_id, hard=False):
        self['recycled_articles'].append(doc_id)
        self['private_articles'].remove(doc_id)
        if hard:
            self.delete_recycled_article(doc_id)

    def delete_public_article(self, doc_id, hard=False):
        self['recycled_articles'].append(doc_id)
        self['public_articles'].remove(doc_id)
        if hard:
            self.delete_recycled_article(doc_id)

    def is_own_article(self, doc_id):
        state = self._get_article_state(doc_id)
        if state == 3:
            return False
        return True

    def _get_article_state(self, doc_id):
        if doc_id in self['private_articles']:
            return 0
        elif doc_id in self['public_articles']:
            return 1
        elif doc_id in self['recycled_articles']:
            return 2
        else:
            return 3


class Document(DefaultDict, PointDict):
    default = dict(
        id=lambda: uuid.uuid4().hex,
        title=None,
        author=None,
        content=None,
        digest=None,
        keywords=None,
        category=None,
        tags=None,

        topic=None,
        introduction=None,
        picture=None,
        url=None,
        filename=None,

        click=None,
        view=None,
        zan=None,
        cai=None,
        created=time.time,
        last_edit=time.time,
        contributors=None,
        links=None,

        contentType=None,
        html=None,
        text=None,
        data=None,
    )
    info_fields = list(filter(lambda key: key not in ['content', 'html', 'text', 'data'], default.keys()))

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
    def gen_search_context(self):
        searchable_keys=[
            'title','content','text','digest','keywords','category','tags','topic','introduction',
        ]
        context=''
        for key in searchable_keys:
            v=self.get(key,None)
            if v:
                if isinstance(v,(list,tuple,set)):
                    context+=';'.join([str(x) for x in v])+';'
                else:
                    context+=str(v)+';'
        return context




    def info(self):
        return {k: self[k] for k in self.info_fields}

    def set_default(self, name, value):
        if name not in self.keys() or self[name] is None or self[name]=='':
            self[name] = value

    def check_fields(self):
        assert self['html']
        assert self['text']
        assert self['title']

        t = time.time()
        self.set_default('created', t)
        self.set_default('last_edit', t)
        self.set_default('content', self['html'])
        use_textrank=False
        if not use_textrank:
            self.set_default('digest', self['text'][:min(100, len(self['text']))])
        else:
            tr4s=TextRank4Sentence()
            if not  self.get('digest',None):
                tr4s.analyze(text=self['text'])
                # print(tr4s.get_key_sentences(num=3))
                self['digest']=';'.join([x['sentence'] for x in tr4s.get_key_sentences(num=3)])
        self['digest'] = self['digest'][:min(100, len(self['digest']))]
        return self


class DocumentStorage:
    '''
    Worries:
        function id2url is given when called, once it changed, all the articles' urls are changed,
        then those references would become invalid.

    '''

    def __init__(self, path, id2url, update_when_start=True):
        self.id2url = id2url
        '''用于生成文章链接'''
        if not os.path.exists(path):
            os.makedirs(path)
        assert os.path.isdir(path)
        self.path = path
        dbpath = path + '/db'
        files_dir = path + '/files'
        self.db = db.Piu(dbpath)
        self.files_dir = Folder(files_dir)
        if update_when_start:
            self.update_all_article_fields()
    def update_all_article_fields(self):
        for k, v in self.db.dic.items():
            # print(k,v)
            if not 'filename' in v.keys() or not v['filename']:
                filename = k
                print('warning***: no filename for %s' % (k))
            else:
                filename = v['filename']
            with self.files_dir.open(filename, 'r') as f:
                doc = json.load(f)
                doc['id'] = k
                doc['filename'] = filename
                doc['url'] = self.id2url(doc['id'])
                doc = Document(**doc)

            with self.files_dir.open(filename, 'w') as f:
                json.dump(doc, f)
            info = doc.info()
            self.db.set(info['id'], info)

    def check(self, doc):
        def check_field(doc, name):
            if not name in doc.keys() or not doc[name]:
                return False
            return True

        if not check_field(doc, 'title'):
            return wk.StatusError(message='标题不能为空')
        if not check_field(doc, 'html'):
            return wk.StatusError(message='正文不能为空')
        if not check_field(doc, 'text'):
            return wk.StatusError(message='正文不能为空')
        return wk.StatusSuccess()

    def delete(self, doc_id, hard=False):
        assert self.db.exists(doc_id)
        info = self.db.get(doc_id)
        self.db.delete(doc_id)
        if hard:
            fn = info['filename']
            self.files_dir.remove(fn)
        return info

    def update_document(self, id, doc):
        document = self.get(id)
        document.update(**doc)
        document.update(keywords=get_keywords(document['text']))
        document = Document(**document).check_fields()
        info = document.info()
        with self.files_dir.open(id, 'w') as f:
            json.dump(document, f, ensure_ascii=False, indent=2)
        self.db.set(id, info)
        return info

    def save(self, doc):
        id = uuid.uuid4().hex
        doc.update(
            id=id, keywords=get_keywords(doc['text']),
            filename=id, url=self.id2url(id)
        )
        doc = Document(**doc).check_fields()
        info = doc.info()
        with self.files_dir.open(id, 'w') as f:
            json.dump(doc, f, ensure_ascii=False, indent=2)
        self.db.set(id, info)
        return info

    def get_info(self, doc_id):
        return self.db.get(doc_id, None)

    def get(self, doc_id, default=None):
        if doc_id in self.db.dic.keys():
            filename = self.db.get(doc_id)['filename']
            with self.files_dir.open(filename, 'r') as f:
                return Document(**json.load(f))
        return default




class DocumentStorageEngine(DocumentStorage):
    def __init__(self, path, index_engine, usman, id2url, update_when_start=True):
        from wk import web
        assert isinstance(usman, web.UserManager)
        assert isinstance(index_engine, IndexEngine)
        super().__init__(path, id2url, update_when_start)
        self.index_engine = index_engine
        self.usman = usman
        self.recycle_bin = db.Piu(self.path + '/recycleBinDB')
        self.doc_market = db.Piu(self.path + '/publicDocumentsDB')

    def get_public(self, doc_id):
        if doc_id in self.doc_market.keys():
            return self.get(doc_id, None)
        return None

    def publish(self, user, doc_id):
        assert isinstance(user, User)
        self.doc_market.add(doc_id, doc_id)
        user.publish_article(doc_id)
        self.usman.update_user(user['id'], user)
        doc = self.get(doc_id)
        assert isinstance(doc,Document)
        self.index_engine.add_document(doc_id, doc.gen_search_context())
        return self.id2url(doc_id)

    def new(self, user, doc):
        assert isinstance(user, User)
        doc['author'] = user['id']
        info = self.save(doc)
        user.new_article(info['id'])
        self.usman.update_user(user['id'], user)
        return info

    def update(self, id, doc):
        return self.update_document(id, doc)

    def is_private(self, doc_id):
        return self.article_status(doc_id) == 0

    def is_public(self, doc_id):
        return self.article_status(doc_id) == 1

    def is_recycled(self, doc_id):
        return self.article_status(doc_id) == 2

    def article_status(self, doc_id):
        if self.doc_market.exists(doc_id):
            return 1
        elif self.recycle_bin.exists(doc_id):
            return 2
        elif self.db.exists(doc_id):
            return 0
        else:
            return 3

    def delete_article(self, user, doc_id, hard=False):
        assert isinstance(user, User)
        state = user._get_article_state(doc_id)
        if state == 0:
            return self.delete_private(user, doc_id, hard=hard)
        elif state == 1:
            return self.delete_public(user, doc_id, hard=hard)
        elif state == 2:
            return self.delete_recycled(user, doc_id)
        else:
            return 'Article Not Found.'

    def delete_recycled(self, user, doc_id):
        assert isinstance(user, User)
        self.recycle_bin.delete(doc_id)
        info = self.db.delete(doc_id)
        DocumentStorage.delete(self, doc_id, hard=True)
        user.delete_recycled_article(doc_id)
        self.usman.update_user(user['id'], user)
        return info

    def delete_public(self, user, doc_id, hard=False):
        assert isinstance(user, User)
        doc = self.get(doc_id)
        assert doc
        self.index_engine.remove_document(doc_id)
        if hard:
            self.doc_market.delete(doc_id)
            DocumentStorage.delete(self, doc_id, hard=True)
            user.delete_public_article(doc_id, hard=True)
        else:
            self.doc_market.delete(doc_id)
            self.recycle_bin.add(doc_id, doc_id)
            user.delete_public_article(doc_id, hard=False)
        self.usman.update_user(user['id'], user)

    def delete_private(self, user, doc_id, hard=False):
        assert isinstance(user, User)
        if hard:
            DocumentStorage.delete(self, doc_id, hard=True)
            user.delete_private_article(doc_id, hard=True)
        else:
            self.recycle_bin.add(doc_id, doc_id)
            user.delete_private_article(doc_id, hard=False)
        self.usman.update_user(user['id'], user)
    def topk_public(self,k):
        return sorted(list(self.doc_market.keys()),key=lambda key:self.db.get(key)['created'],reverse=True)[:k]

class FileDescriptor(DefaultDict):
    default = dict(
        id=lambda: uuid.uuid4().hex,
        original_filename=None,
        filename=None,
        name=lambda this: os.path.basename(this['original_filename']),
        keywords=None,
        absolute_filepath=None,
        relative_filepath=None,
        type='file',
        description=None,
        category=None,
        tags=None,
        topic=None,
        provider=None,
        owner=None,
        permission=None,
        size=None,
        mimetype=None,
        create_time=None,
        likes=0,
        dislikes=0,
        zan=0,
        cai=0,
        view=0,
    )

    def __init__(self, **kwargs):
        super().__init__(**kwargs)


class StaticStorageEngine(Folder):
    def __init__(self, path, check_when_start=False, include_dirs=[]):
        super().__init__(path)
        self.dbpath = path + '/db'
        self.db = db.Piu(self.dbpath)
        self.files_dir = Folder(self.path + '/file')
        self.images_dir = Folder(self.path + '/image')
        self.type2folder = {
            'file': self.files_dir,
            'image': self.images_dir,
        }
        self.include_dirs = include_dirs
        self.track_db = db.Piu(self.path + '/track_db')
        if check_when_start:
            self.startup()
            self.track_include_dirs()

    def track_include_dirs(self):
        def get_subject(tags):
            subjects = ['语文', '数学', '英语', '物理', '化学', '生物', '政治', '历史', '地理']
            res=list(set(tags).intersection(set(subjects)))
            return res[0] if res else None

        for dir in self.include_dirs:
            folder = Folder(dir)
            print("Finding files...")
            fs = folder.glob('/**/*.*', recursive=True)
            self.db.pause_save()
            self.track_db.pause_save()
            for i, f in enumerate(fs):

                fn = os.path.basename(f)
                absolute_filepath = folder._truepath(f)
                tags = jieba.analyse.extract_tags(f, topK=10)
                subject=get_subject(tags)
                if self.track_db.exists(absolute_filepath):
                    id = self.track_db.get(absolute_filepath)
                    self.db.delete(id)
                    print('delete %s' % id)
                    # continue
                print('including:', absolute_filepath)
                fd = FileDescriptor(
                    filename=fn,
                    original_filename=fn,
                    absolute_filepath=absolute_filepath,
                    description=f,
                    category=os.path.dirname(f),
                    keywords=tags,
                    tags=tags,
                    subject=subject,
                )
                self.db.set(fd['id'], fd)
                self.track_db.set(absolute_filepath, fd['id'])
            self.db.resume_save(save_now=True)
            self.track_db.resume_save(save_now=True)

    def startup(self):
        self.db.pause_save()
        for k in list(self.db.keys()):
            v = self.db.get(k)
            v = FileDescriptor(**v)
            self.db.set(k, v)
            if ( not v['relative_filepath'] )and not  v['absolute_filepath']:
                self.db.delete(k)
                print('delete file %s'%k)
            elif v['relative_filepath'] and not self.exists( v['relative_filepath']):
                self.db.delete(k)
                print("File %s not exists. delete the record."%(v['relative_filepath']))
            elif v['absolute_filepath'] and not os.path.exists( v['absolute_filepath']):
                self.db.delete(k)
                print("File %s not exists. delete the record."%(v['absolute_filepath']))


        self.db.resume_save()
        # self.db.pprint()
    def update_file(self,id,file):
        info=self.db.get(id)
        info.update(**file)
        self.db.set(id,info)

    def send_file(self, id):
        info = self.db.get(id)
        if info:
            if info['absolute_filepath']:
                return web.send_file(info['absolute_filepath'])
            return web.send_file(self._truepath(info['relative_filepath']))
        else:
            return web.StatusErrorResponse(message='File Not Found.').jsonify()

    def get_file_info(self, id):
        info = self.db.get(id)
        # print(info)
        if not info['absolute_filepath']:
            # print(info['relative_filepath'])
            info['absolute_filepath'] = self._truepath(info['relative_filepath'])
        info.update(get_file_info(info['absolute_filepath']))
        return info
    def get(self,id,default=None):
        return self.db.get(id,default)
    def saveImage(self, file):
        return self._saveFile(file, dict(type='image'))

    def saveFile(self, file):
        return self._saveFile(file, dict(type='file'))

    def _saveFile(self, file, info={}):
        '''file:
        filename , name , save , mimetype , close
        '''
        type = info.get('type', 'file')
        id = uuid.uuid4().hex
        original_filename = file.filename
        mimetype = file.mimetype
        filename = '%s-%s' % (id, original_filename)
        folder = self.type2folder[type]
        relative_path = folder.name + '/' + filename
        absolute_path = folder._truepath(filename)
        folder.save_http_file(file, filename)
        fd = FileDescriptor(
            id=id,
            type=type,
            mimetype=mimetype,
            filename=filename,
            original_filename=original_filename,
            relative_filepath=relative_path,
            absolute_filepath=absolute_path,
        )
        fd.update(**folder.info(filename))
        self.db.set(id, fd)
        return fd

    def select(self, func=None, **kwargs):
        return self.db.select(func=func, **kwargs)

    def search(self, q, match_all=True, candidates=None):
        assert isinstance(candidates, (list, tuple))
        query = []
        for word in q.split():
            query += list(jieba.cut(word))

        def get_search_context(id):
            file = self.get_file_info(id)
            return '\t'.join([str(v) if v is not None else '' for v in file.values()])

        files = candidates if not candidates is None else self.db.keys()
        if match_all:

            for word in query:
                files = list(filter(lambda id: re.findall(word, get_search_context(id)), files))
            return files
        else:
            def match(text, ptns):
                for ptn in ptns:
                    if re.findall(ptn, text): return True
                return False

            files = list(filter(lambda id: match(get_search_context(id), query), files))
            return files


class IndexEngine:
    def __init__(self, path):
        if not os.path.exists(path):
            os.makedirs(path)
        dbpath = path + '/db'
        self.db = Piu(dbpath)
        self.path = path
        self.inverted, self.doc_num, self.idf, self.id2doc = self.restore()

    def restore(self):
        inverted = self.db.get('inverted', None)
        doc_num = self.db.get('doc_num', None)
        idf = self.db.get('idf', None)
        id2doc = self.db.get('id2doc', None)
        if inverted is None:
            inverted = {}
            doc_num = 0
            idf = {}
            self.db.set('inverted', inverted)
            self.db.set('doc_num', doc_num)
            self.db.set('idf', idf)
            self.db.set('id2doc', id2doc)
        return inverted, doc_num, idf, id2doc

    def save_all(self):
        self.db.set('inverted', self.inverted)
        self.db.set('doc_num', self.doc_num)
        self.db.set('idf', self.idf)
        self.db.set('id2doc', self.id2doc)

    def remove_document(self, doc_id, save=True):
        for term in self.inverted.keys():
            if doc_id in self.inverted[term]:
                self.inverted[term].pop(doc_id)
        self.doc_num -= 1
        for term in list(self.idf.keys()):
            if not self.doc_num:
                self.idf.pop(term)
                continue
            self.idf[term] = math.log10(self.doc_num / (len(self.inverted[term]) + 1e-10))
        if save:
            self.save_all()

    def remove_document_batch(self, doc_ids: list, save=True):
        for doc_id in doc_ids:
            for term in self.inverted.keys():
                if doc_id in self.inverted[term]:
                    self.inverted[term].remove(doc_id)
            self.doc_num -= 1
        self.compute_idf(save=False)
        if save:
            self.save_all()

    def compute_idf(self, save=True):
        for term in self.idf.keys():
            self.idf[term] = math.log10(self.doc_num / len(self.inverted[term]))
        if save:
            self.save_all()

    def add_document(self, doc_id, text, save=True):
        terms = []
        for txt in text.split():
            terms += list(jieba.cut_for_search(txt))
        for term in terms:
            if term in self.inverted.keys():
                if doc_id in self.inverted[term].keys():
                    self.inverted[term][doc_id] += 1
                else:
                    self.inverted[term][doc_id] = 1
            else:
                self.inverted[term] = {doc_id: 1}
        self.doc_num += 1
        for term in terms:
            self.idf[term] = math.log10(self.doc_num / len(self.inverted[term]))
        if save:
            self.save_all()


class SearchEngine:
    def __init__(self, index_engine):
        self.index_engine = index_engine

    def search(self, query):
        doc_id = self._search_doc(query)
        return doc_id

    def _search_doc(self, query):
        terms = []
        for txt in query.split():
            terms += list(jieba.cut_for_search(txt))
        tf_idf = {}
        for term in terms:
            if term in self.index_engine.inverted.keys():
                for doc_id, freq in self.index_engine.inverted[term].items():
                    score = 1 + math.log10(freq) * self.index_engine.idf[term]
                    if doc_id in tf_idf.keys():
                        tf_idf[doc_id] += score
                    else:
                        tf_idf[doc_id] = score
        doc_scores = sorted(list(tf_idf.items()), key=lambda doc: doc[1], reverse=True)
        result = [doc_id for doc_id, score in doc_scores]
        return result

class RecommenderSystem:
    '''
    调用document_engine获取推荐度最高的文章
    '''
    def __init__(self,document_engine):
        assert isinstance(document_engine,DocumentStorageEngine)
        self.document_engine=document_engine
    def recommend_latest(self,n=20):
        return self.document_engine.topk_public(k=n)



from wk.io import db
import wk
from wk.basic import join_path, IterObject, SecureDirPath, PointDict, Path, DirPath, PowerDirPath, Status, \
    StatusSuccess, StatusError
from flask import request, render_template, redirect, make_response, jsonify
import functools, inspect
from jinja2 import Environment, PackageLoader

usman_env = Environment(loader=PackageLoader('wk.data', 'templates/modules/UserManager'))
import inspect,uuid,copy


def log_func(msg="*** running %s ...."):
    # def decorator(func):
    #     @functools.wraps(func)
    #     def wrapper(*args,**kwargs)
    #         print(msg%(func.__name__) if "%s" in msg else msg)
    #         func(*args,**kwargs)
    #     return wrapper
    def before(func):
        print(msg % (func.__name__) if "%s" in msg else msg)

    decorator = config_run(before=before)
    return decorator


def config_run(before=None, after=None):
    def decorator(func):
        def do_before():
            dosome = before
            if not dosome: return
            if hasattr(dosome, '__call__'):
                dosome_args = inspect.getfullargspec(dosome).args
                if 'func' in dosome_args:
                    dosome(func=func)
                else:
                    dosome()
            else:
                print(dosome)

        def do_after():
            dosome = after
            if not dosome: return
            if hasattr(dosome, '__call__'):
                dosome_args = inspect.getfullargspec(dosome).args
                if 'func' in dosome_args:
                    dosome(func=func)
                else:
                    dosome()
            else:
                print(dosome)

        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            do_before()
            res = func(*args, **kwargs)
            do_after()
            return res

        # print("wrapper args:",inspect.getfullargspec(wrapper).args)
        # print("func args:",inspect.getfullargspec(func).args)
        return wrapper

    return decorator


def rename_func(name):
    def decorator(func):
        func.__name__ = name

        @functools.wraps(func)
        def new_func(*args, **kwargs):
            return func(*args, **kwargs)

        return new_func

    return decorator


def parse_from(*refers):
    def decorator(f):
        fargs = inspect.getfullargspec(f).args

        @functools.wraps(f)
        def wrapper(*args, **kwargs):
            dic = {}
            for ref in refers:
                d = ref() if callable(ref) else dict(ref)
                d = d or {}
                if d: dic.update(d)
            params = {}
            for ag in fargs:
                params[ag] = dic.get(ag, None)
                if params[ag] is None:
                    for k, v in dic.items():
                        if k.replace('-', '_') == ag:
                            params[ag] = v
            # print("args:",fargs)
            # print("params:",params)
            params.update(kwargs)
            return f(*args, **params)

        return wrapper

    return decorator


def get_files(): return dict(request.files)


def get_form(): return request.form


def get_json(): return request.json


def get_cookies(): return request.cookies


def get_url_args(): return request.args


# parse_json is a decorator
parse_json_and_form = parse_from(get_json, get_form)
parse_json = parse_from(get_json)
parse_form = parse_from(get_form)
parse_files = parse_from(get_files)
parse_cookies = parse_from(get_cookies)
parse_args = parse_from(get_url_args)
parse_all = parse_from(get_cookies, get_form, get_json, get_url_args)


class Context(PointDict):
    def __init__(self,*args,**kwargs):
        super().__init__(*args,**kwargs)
        self.utils=PointDict(
            wk=wk
        )

def log(*msgs):
    print("log".center(10, '*') + ":" + ' '.join([str(msg) for msg in msgs]))



class UserManager:
    '''
    user:
        id
        username
        email
        password
        articles
        zans
        cais
        views
        picture
        signature
        introduction

    '''
    __status_succeeded__ = 'succeeded'
    __status_failed__ = 'failed'

    def __init__(self, dbpath,User, home_url='/', url_prefix='/', signup_url=None, login_url=None,logout_url=None):
        self.db = db.Piu(dbpath)
        self.User = User
        self.home_url = home_url
        self.url_prefix = url_prefix
        self.signup_url = signup_url or join_path(url_prefix, 'join')
        self.login_url = login_url or join_path(url_prefix, 'login')
        self.logout_url = logout_url or join_path(url_prefix, 'logout')
        self.update_all_user_fields()
    def register(self,app):
        app.route(self.signup_url, methods=['get', 'post'])(self.signup())
        app.route(self.login_url, methods=['get', 'post'])(self.login())
        app.route(self.logout_url, methods=['get', 'post'])(self.logout())
    def exists_user(self, email):
        if not self.get_user(email): return False
        return True
    def users(self):
        return self.db.keys()

    def update_all_user_fields(self):
        self.db.pause_save()
        for k, v in self.db.dic.items():
            v=self.User(**v)
            self.db.set(k,v)
        self.db.resume_save(save_now=True)
    def add_user(self,user={}):
        user=self.User(**user)
        id=user['id']
        self.db.add(id,user)
        return id

    def get_user(self, id):
        return copy.deepcopy(self.db.get(id, None))

    def set_user(self,id,user):
        self.db.set(id,user)
    def update_user(self, id, info={}):
        if not self.get_user(id):
            self.add_user(id)
        user=self.db.get(id)
        user.update(**info)
        self.db.set(id,user)

    def status(self, status, **kwargs):
        return jsonify(dict(status=status, **kwargs))

    def home_page(self, **kwargs):
        return redirect('/')

    def redirect_page(self,target,target_text=None,message=None,source=None):
        return usman_env.get_template('Redirect.tem').render(target=target,target_text=target_text,message=message,source=source)

    def signup_page(self, target, method='post', **kwargs):
        return usman_env.get_template('SignUp.tem').render(action=target, method=method, **kwargs)

    def login_page(self, target, method='post', **kwargs):
        return usman_env.get_template('Login.tem').render(action=target, method=method, **kwargs)

    def error_page(self, **kwargs):
        return usman_env.get_template('Error.tem').render(**kwargs)

    def get_user_context(self,arg_name='user'):
        '''从cookies中获取用户信息'''
        def decorator(f):
            @functools.wraps(f)
            @parse_cookies
            def wrapper(email, username, password, *args, **kwargs):
                # log(email,username,password)
                user=self.check_user(email,username,password)
                if not isinstance(user,self.User):
                    user= None
                kwargs.update(**{arg_name:user})
                return f(*args,**kwargs)
            return wrapper
        return decorator
    def check_user(self,email,username,password):
        if not (email and password) and not (username and password):
            return None
        if email:
            user = self.db.search(email=email)
            if not user:
                return self.redirect_page(target=self.signup_url,target_text='登录页面',message='请注册')
                # return self.signup_page(target=self.home_url, method='post')
            user=user[0]
            if user and (user['email'] == email) and (user['password'] == password):
                return user
            else:
                return None
        if username:
            user = self.db.search(username=username)
            user = PointDict.from_dict(user) if user else user
            if not user:
                return None
            if user and (user.username == username) and (user.password == password):
                return user
            else:
                return self.error_page()

    def login_required(self, f):
        @functools.wraps(f)
        @parse_cookies
        def wrapper(email, username,password, *args, **kwargs):
            # print(email,username,password)
            if not (email and password) and not(username and password):
                return self.redirect_page(target=self.signup_url,target_text='登录页面', message='请先登录')
                # return self.login_page(target=self.login_url,method='post')
            if email:
                user = self.db.search(email=email)
                if not user:
                    return self.redirect_page(target=self.signup_url,target_text='注册页面',message='请注册')
                    # return self.signup_page(target=self.signup_url, method='post')
                user=user[0]
                if user and (user['email'] == email) and (user['password'] == password):
                    return f(*args, **kwargs)
                else:
                    return self.error_page()
            elif username:
                user = self.db.search(username=username)
                if not user:
                    return self.redirect_page(target=self.signup_url,target_text='注册页面', message='请注册')
                    # return self.signup_page(target=self.signup_url, method='post')
                if user:
                    user=user[0]
                if user and (user['username'] == username) and (user['password'] == password):
                    return f(*args, **kwargs)
                else:
                    return self.error_page()
        return wrapper

    def signup(self):
        @parse_form
        def do_signup(email, password, username):
            if not email:
                return self.signup_page(target=self.signup_url, method='post')
            log("sign up:", email, password, username)
            if self.db.search(email=email):
                msg = "Email has been taken"
                log(msg)
                return jsonify(StatusError(message=msg))
            if self.db.search(username=username):
                msg = "Username has been taken"
                log(msg)
                return jsonify(StatusError(message=msg))

            id=self.add_user({'email': email, 'password': password, "username": username})
            log(self.db.get(id))
            resp = redirect(self.home_url)
            resp.set_cookie('email', email)
            resp.set_cookie('username', email)
            resp.set_cookie('password', password)
            return resp

        return do_signup
    def logout(self):
        @log_func()
        @parse_form
        def do_logout(email, username, password):
            resp = redirect(self.home_url)
            resp.set_cookie('email', '',expires=0)
            resp.set_cookie('username', '',expires=0)
            resp.set_cookie('password', '',expires=0)
            log("resp:", resp)
            return resp
        return do_logout
    def login(self):
        @log_func()
        @parse_form
        def do_login(email,username, password):
            log( email, username, password)
            if not email and not username:
                return self.login_page(target=self.login_url, method='post')
            if email:
                if not self.db.search(email=email):
                    msg = "Email doesn't exists."
                    print(msg)
                    return self.status(self.__status_failed__, msg=msg)
                # resp = make_response(self.home_page()) if not redirect_to else redirect_to
                resp = redirect(self.home_url)
            else:
                assert username
                if not self.db.search(username=username):
                    msg = "Username doesn't exists."
                    print(msg)
                    return self.status(self.__status_failed__, msg=msg)
                # resp = make_response(self.home_page()) if not redirect_to else redirect_to
                resp = redirect(self.home_url)
            resp.set_cookie('email', email)
            resp.set_cookie('password', password)
            log("resp:", resp)
            return resp

        return do_login



'''
dependencies:
linux: wget lsof
'''

import os,subprocess,time,shutil
test_link= 'https://dn3.freedownloadmanager.org/5/5.1-latest/fdm5_x64_setup.exe'
test_git_link='https://github.com/myhub/tr'
def shell_cmd(cmd):
    p=subprocess.Popen(cmd,shell=True)
    p.wait()
    return p.returncode
def chmod(mode,dst):
    return shell_cmd('chmod %s %s'%(mode,dst))
def clean_port(port=8000): # prerequisite: lsof
    os.system('kill -9 `lsof -i:%s -t`'%(port))
def wget_download(url,out_dir=None,log_file=None): # prerequisite: wget
    dir_str,log_str='',''
    if out_dir is not  None:
        dir_str='-P '+out_dir
    if log_file is not  None:
        log_str = ' > ' + log_file + ' 2>&1 &'
    cmd = 'wget %s %s %s'%(dir_str,url,log_str)
    p=subprocess.Popen(cmd,shell=True)
    return p
def git_clone(url,out_dir=''):
    cmd='git clone %s %s'%(url,out_dir)
    p=subprocess.Popen(cmd,shell=True,)
    p.wait()
    return p.returncode
def zip_dir(src,dst=None):
    dst = dst or src+'.zip'
    cmd='zip -rl %s %s'%(dst,src)
    r=shell_cmd(cmd)
    return r
def add_bash_script_as_service(bpath):
    name=os.path.basename(bpath)
    cmd='rm -f /etc/init.d/%s;cp %s /etc/init.d/;cd /etc/init.d/;chmod +x %s;\
     chkconfig --add %s;chkconfig %s on'%(name,bpath,name,name,name)
    return  shell_cmd(cmd)
def add_py_to_service(service_name,project_dir,script_file_path):
    project_dir=os.path.abspath(project_dir)
    script_file_path=os.path.abspath(script_file_path)
    from wk.extra.gen_scripts import gen_scripts
    gen_scripts.gen_service(dst_file=project_dir+'/'+service_name,service_name=service_name,
                            project_dir=project_dir,script_file_name=script_file_path)
    add_bash_script_as_service(project_dir+'/'+service_name)

if __name__ == '__main__':
    # git_clone(git_link)
    # d=os.path.basename(git_link)
    # zip_dir(d,'tr.zip')
    # add_bash_file_service('/root/projects/mysite/mysite')
    add_py_to_service(service_name='mysite',project_dir='/root/projects/mysite',script_file_path='main.py')
    pass


import argparse
import os
os.environ['ANSI_COLORS_DISABLED']="1"
import wk

def run_default():
    words = 'Hello, I am wpkit2 ,use me !'
    length = 50
    print('*' * length)
    print(words.center(length, '*'))
    print('*' * length)
def main_bold():
    parser = argparse.ArgumentParser()
    parser.add_argument('-command', type=str)
    args = parser.parse_args()
    if args.command is None:
        run_default()
    elif args.command=='deploy':
        pass
def main():
    import fire
    class Cli:
        @staticmethod
        def hi():
            run_default()

        @staticmethod
        def downtee( key, path=None, overwrite=False):
            from wk.extra.gitspace import Store
            store = Store()
            store.get(key, path=path, overwrite=overwrite)

        @staticmethod
        def uptee( key, path, recursive=False):
            from wk.extra.gitspace import Store
            store = Store()
            store.set(key, path, recursive)

        @staticmethod
        def deploy(service,*args,**kwargs):
            if service=='fsapp':
                from wk.applications import fsapp
                fsapp.setup_default(*args,**kwargs)
            elif service=='zspt':
                from wk.applications import zspt
                zspt.setup_default(*args,**kwargs)
            else:
                print("Service %s is not valid."%(service))
    fire.Fire(Cli)

if __name__ == '__main__':
    main()



from flask import Flask, request, Blueprint, abort, send_file
from wpkit.web import resources,utils
from wpkit.pan import Pan
from wpkit.web.base import MyBlueprint
from wpkit.web.resources import env
import wpkit

class BlueStatic(MyBlueprint):
    def __init__(self,import_name=None,name='static',url_prefix='/',static_dir='./',template=None,debug=False,**kwargs):
        super().__init__(name=name,import_name=import_name,url_prefix=url_prefix,**kwargs)
        def tprint(*args, **kwargs):
            if debug:
                print('**info:', *args, **kwargs)
        import os
        template = resources.default_templates['files'] if not template else template
        @self.route('/', defaults={'req_path': ''})
        @self.route(utils.join_path('/', '<path:req_path>'))
        def dir_listing(req_path):
            tprint('req_path:',req_path)
            BASE_DIR = static_dir
            # print(static_dir)
            abs_path = os.path.join(BASE_DIR, req_path)
            abs_path=os.path.abspath(abs_path)
            # print(abs_path)
            if not os.path.exists(abs_path):
                return abort(404)
            if os.path.isfile(abs_path):
                # print(abs_path)
                return send_file(abs_path)
            if os.path.isdir(abs_path):
                fns = os.listdir(abs_path)
                fps = [utils.join_path(self.url_prefix,url_prefix, req_path, f) for f in fns]
                return utils.render(open(template, 'r', encoding='utf-8').read(), files=zip(fps, fns))

if __name__ == '__main__':
    BlueStatic(__name__).run()


import json, os, shutil
# from collections import deque
# from uuid import uuid4
from wk.basic import T, PointDict, PowerDirPath, Status, StatusSuccess, StatusError
from ..ioutils import json_load, json_dump


class Piu:
    def __init__(self, path='./db',remake=False):
        if remake and os.path.exists(path):
            shutil.rmtree(path)
        self.dbpath = path
        self.dicpath = os.path.join(self.dbpath, 'data.dic')
        self.dic = self.setup()
        self.pause_save_flag = False

    def pprint(self):
        import pprint
        print(self.dic)

    def pause_save(self):
        self.pause_save_flag = True

    def resume_save(self, save_now=False):
        self.pause_save_flag = False
        if save_now:
            self._save()

    def setup(self):
        if self._exists(): return json_load(self.dicpath)
        return self._make()

    def keys(self):
        return self.dic.keys()

    def values(self):
        return self.dic.values()

    def items(self):
        return self.dic.items()

    def add(self, *args, **kwargs):
        assert len(args) == 0 or len(args) == 2
        if len(args):
            assert isinstance(args[0], str)
            kwargs.update({args[0]: args[1]})
        self.dic.update(**kwargs)
        self._save()

    def set(self, *args, **kwargs):
        assert len(args) == 0 or len(args) == 2
        if len(args):
            assert isinstance(args[0], str)
            kwargs.update({args[0]: args[1]})
        self.dic.update(**kwargs)
        self._save()

    def delete(self, key):
        value = self.dic.pop(key)
        self._save()
        return value

    def _match(self, dic, **kwargs):
        for k, v in kwargs.items():
            if k not in dic.keys():
                return False
            if dic[k] != v:
                return False
        return True

    def select(self,func=None, **kwargs):
        results = []
        for k, v in self.dic.items():
            if isinstance(v, dict) and self._match(v, **kwargs):
                if func and not func(v):
                    continue
                results.append(k)
        return results

    def search(self, with_key=None, **kwargs):
        results = {} if with_key else []
        for k, v in self.dic.items():
            if isinstance(v, dict) and self._match(v, **kwargs):
                if with_key:
                    results[k] = v
                else:
                    results.append(v)
        return results

    def exists(self, key):
        if key in self.dic.keys():
            return True
        else:
            return False

    def get(self, *args, **kwargs):
        return self.dic.get(*args, **kwargs)

    def _save(self):
        if self.pause_save_flag:
            return
        data = json.dumps(self.dic, ensure_ascii=False, indent=2)
        with open(self.dicpath, 'w', encoding='utf-8') as f:
            f.write(data)

    def _exists(self):
        if os.path.exists(self.dbpath) and os.path.exists(self.dicpath):
            return True
        return False

    def _make(self):
        dir = self.dbpath
        shutil.rmtree(dir) if os.path.exists(dir) else None
        os.makedirs(dir)
        dic = {}
        json_dump(dic, self.dicpath)
        return dic


class FileDict(PointDict):
    def __init__(self, path):
        self.seta(path=path)
        path = PowerDirPath(self.geta('path'))
        if path.exists():
            assert path.isfile()
            dic = json_load(path)
            assert isinstance(dic, dict)
        else:
            dic = {}
            path.tofile()
            json_dump(dic, path, indent=4)
        super().__init__(dic)

    def __setattr__(self, key, value):
        PointDict.__setattr__(self, key, value)
        self._save()

    def __setitem__(self, key, value):
        PointDict.__setitem__(self, key, value)
        self._save()

    def update(self, *args, **kwargs):
        for k, v in kwargs.items():
            self[k] = v
        for arg in args:
            self.update(**arg)

    def _save(self):
        json_dump(self, self.geta('path'), indent=4)


class BackupDB(PointDict):
    def __init__(self, path='./db', ignore_duplicated=True, max_depth=10):
        self.dbpath = path
        self.dicpath = os.path.join(self.dbpath, 'data.json')
        self.configfile = os.path.join(self.dbpath, 'config.json')
        self.dic = self.setup()
        self.config = self.setup_configfile()
        self.config.update(ignore_duplicated=ignore_duplicated, max_depth=max_depth)
        self.load_config()

    def setup_configfile(self):
        config = FileDict(self.configfile)
        return config

    def load_config(self):
        for k, v in self.config.items():
            self[k] = v

    def setup(self):
        if self._exists(): return json_load(self.dicpath)
        return self._make()

    def set(self, *args, **kwargs):
        return self.add(*args, **kwargs)

    def add(self, *args, **kwargs):
        assert len(args) == 0 or len(args) == 2
        if len(args):
            assert isinstance(args[0], str)
            kwargs.update({args[0]: args[1]})
        for k, v in kwargs.items():
            if k not in self.dic.keys():
                self.dic[k] = [v]
            elif self.ignore_duplicated and v == self.dic[k][-1]:
                continue
            else:
                self.dic[k].append(v)
                if len(self.dic[k]) > self.max_depth:
                    self.dic[k] = self.dic[k][1:]
        self._save()

    def delete(self, key):
        del self.dic[key]
        self._save()

    def get(self, k, default=T.NOT_GIVEN):
        if k not in self.dic.keys():
            if default == T.NOT_GIVEN:
                raise KeyError('No such key named %s' % (k))
            else:
                return default
        return self.dic[k][-1]

    def recover(self, key, step=1):
        assert key in self.dic.keys()
        assert len(self.dic[key]) > step
        return self.dic[key].pop()

    def _save(self):
        json_dump(self.dic, self.dicpath, indent=4)

    def _exists(self):
        if os.path.exists(self.dbpath) and os.path.exists(self.dicpath):
            return True
        return False

    def _make(self):
        dir = self.dbpath
        shutil.rmtree(dir) if os.path.exists(dir) else None
        os.makedirs(dir)
        dic = {}
        json_dump(dic, self.dicpath, indent=4)
        return dic

    def execute(self, cmd):
        cmd = PointDict.from_dict(cmd)
        op, params = cmd.op, cmd.params
        if op == 'add':
            res = self.add(params['key'], params['value'])
        elif op == 'set':
            res = self.set(params['key'], params['value'])
        elif op == 'get':
            res = self.get(params['key'], params.get('default', None))
        elif op == 'delete':
            res = self.delete(params['key'])
        else:
            assert op == 'recover'
            res = self.recover(params['key'], params['step'])
        return res


RecordClassDict = {}


class RecordMataClass(type):
    def __new__(cls, name, bases, attrs):
        attrs['__record_type__'] = name
        new_cls = type.__new__(cls, name, bases, attrs)
        RecordClassDict[name] = new_cls
        return new_cls


class Record(metaclass=RecordMataClass):
    require_init = False
    require_recover = False

    def __init__(self, dic=None, **kwargs):
        if dic:
            assert isinstance(dic, dict)
            kwargs.update(dic)
        self.seta(dic=kwargs)

    def jsonvalue(self):
        return self.geta('dic')

    def __repr__(self):
        return '%s(%s)' % (self.__class__.__name__, ','.join(['%s=%s' % (k, v) for k, v in self.geta('dic').items()]))

    def todict(self):
        dic = {
            '@is_record': True, '@record_type': self.__record_type__, '@value': self.jsonvalue()
        }
        return dic

    @classmethod
    def fromdict(cls, dic):
        return cls(**dic)

    def keys(self):
        return self.geta('dic').keys()

    def values(self):
        return self.geta('dic').values()

    def items(self):
        return self.geta('dic').items()

    def __getattr__(self, key, default=T.NOT_GIVEN):
        return self.geta('dic').get(key, default)

    def __setattr__(self, key, value):
        self.geta('dic')[key] = value

    def set_attribute(self, key, value):
        self.__dict__[key] = value

    def get_attribute(self, *args, **kwargs):
        return self.__dict__.get(*args, **kwargs)

    def seta(self, **kwargs):
        for k, v in kwargs.items():
            self.set_attribute('__%s__' % (k), v)

    def geta(self, key, *args, **kwargs):
        return self.get_attribute('__%s__' % (key), *args, **kwargs)


class PlainRecord(Record):
    def __init__(self, obj, *args, **kwargs):
        self.seta(obj=obj)
        super().__init__(*args, **kwargs)

    def jsonvalue(self):
        return self.geta('obj')

    def __repr__(self):
        return self.geta('obj').__repr__()

    @classmethod
    def fromdict(cls, dic):
        return cls(dic)


class JsonRecord(Record):
    pass


class LinkRecord(Record):
    def __repr__(self):
        return '%s<%s>' % (self.__class__.__name__, ','.join(['%s=%s' % (k, v) for k, v in self.geta('dic').items()]))


class FileRecord(LinkRecord):
    require_init = True
    require_recover = True

    def __init__(self, fname=None, fid=None, content=None, encoding='utf-8', *args, **kwargs):
        assert fid or content
        kwargs.update(fname=fname, fid=fid, encoding=encoding)
        if content: self.seta(content=content)
        super().__init__(*args, **kwargs)

    def init(self, key):
        self.seta(helper=key)
        content = self.geta('content', None)
        fid = self.fid
        if fid and content:  # write to file
            fp = key.gen_filepath(fid)
            PowerDirPath(fp).tofile()(content, encoding=self.encoding)
        elif fid and not content:  # read file
            assert key.get_filepath(fid)
        elif not fid and content:  # write to file
            self.fid = key.gen_fid()
            fp = key.gen_filepath(self.fid)
            PowerDirPath(fp).tofile()(content, encoding=self.encoding)
        else:  # no file, no content
            raise Exception('Error:File record needs fid or content to be given.')

    def recover(self, helper):
        self.seta(helper=helper)

    def __call__(self, *args, **kwargs):
        helper = self.geta('helper')
        fp = helper.get_filepath(self.fid)
        # print(fp)
        if fp: return PowerDirPath(fp)(*args, **kwargs)


class ImageRecord(FileRecord):
    pass


def record_default(obj):
    return obj.todict() if 'todict' in dir(obj) else obj


def record_hook(dic):
    for key, value in dic.items():
        if isinstance(value, dict):
            if value.get('@is_record', None) and value.get('@record_type', T.NOT_FOUND) != T.NOT_FOUND:
                type = value['@record_type']
                assert type in RecordClassDict.keys()
                cls = RecordClassDict[type]
                new_value = cls.fromdict(value['@value'])
                dic[key] = new_value
    return dic


class FileStorageHelper(Piu):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.filesdir = PowerDirPath(os.path.join(self.dbpath, 'files'))
        self.filesdir.todir()
        # print('piu build up...:\nfids:',self.get('fids:'))
        if not self.get('fids', None): self.add(fids=[])
        if self.get('first_fid', T.NOT_EXISTS) == T.NOT_EXISTS: self.add(first_id='0')
        if not self.get('last_fid', None): self.add(last_fid=self.get('first_fid'))

    def add_fid(self, fid):
        fids = self.get('fids')
        fids.append(fid)
        self.add(last_fid=fid)
        self.add(fids=fids)

    def gen_fid(self):
        last_fid = int(self.get('last_fid'))
        last_fid = str(last_fid + 1)
        self.add(last_fid=last_fid)
        self.add_fid(last_fid)
        # print('new fids:',self.get('fids'))
        return last_fid

    def get_filepath(self, fid):
        # print(self.get('last_fid'))
        return self.filesdir / fid if fid in self.get('fids') else None

    def gen_filepath(self, fid):
        self.add_fid(fid) if not fid in self.get('fids') else None
        return self.filesdir / fid


class Table(PointDict):
    def __init__(self, path='./db'):
        self.dbpath = path
        self.dicpath = os.path.join(self.dbpath, 'data.json')
        self.configfile = os.path.join(self.dbpath, 'config.json')
        self.dic = self.setup()
        self.config = self.setup_configfile()
        self.config.update()
        self.load_config()
        self.helper = FileStorageHelper(os.path.join(self.dbpath, 'FileStorageHelper'))

    def setup_configfile(self):
        config = FileDict(self.configfile)
        return config

    def load_config(self):
        for k, v in self.config.items():
            self[k] = v

    def setup(self):
        if self._exists(): return json_load(self.dicpath, object_hook=record_hook)
        return self._make()

    def add(self, *args, **kwargs):
        assert len(args) == 0 or len(args) == 2
        if len(args):
            assert isinstance(args[0], str)
            kwargs.update({args[0]: args[1]})
        for k, v in kwargs.items():
            if not isinstance(v, Record): v = PlainRecord(v)
            if v.require_init:  # for example: FileRecord needs to be initialized.
                v.init(self.helper)
            self.dic[k] = v
        self._save()

    def set(self, *args, **kwargs):
        return self.add(*args, **kwargs)

    def delete(self, key):
        del self.dic[key]
        self._save()

    def get(self, k, default=T.NOT_GIVEN):
        if k not in self.dic.keys():
            if default == T.NOT_GIVEN:
                raise KeyError('No such key named %s' % (k))
            else:
                return default
        v = self.dic[k]
        if v.require_recover: v.recover(self.helper)
        return v

    def _save(self):
        json_dump(self.dic, self.dicpath, indent=4, default=record_default)

    def _exists(self):
        if os.path.exists(self.dbpath) and os.path.exists(self.dicpath):
            return True
        return False

    def _make(self):
        dir = self.dbpath
        shutil.rmtree(dir) if os.path.exists(dir) else None
        os.makedirs(dir)
        dic = {}
        json_dump(dic, self.dicpath, indent=4)
        return dic


def demo():
    P = Piu()
    P.add('a', 13)
    P.add('name', 'wangpei')
    P.add('age', 21)
    P.delete('a')
    age = P.get('age')
    print(age)
    x = P.get('x', 30)
    print(x)


if __name__ == '__main__':
    demo()



'''
requirements:
Linux with wget installed.
'''
from wpkit.web import utils,resources
from flask import request
from wpkit.web.base import MyBlueprint
import os
class BluePostAndDownload(MyBlueprint):
    add_to_sitemap = True
    url_prefix = '/post_and_download'
    def __init__(self,import_name=None,name='PostAndDownload',data_dir='/var/www/html',url_prefix='/post_and_download',**kwargs):
        self.data_dir = data_dir
        if not os.path.exists(data_dir):
            os.makedirs(data_dir)
        self.download_url = self.url_prefix + '/download'
        super().__init__(name=name,import_name=import_name,url_prefix=url_prefix,**kwargs)
    def register(self, app, options, first_registration=False):
        if not hasattr(app, 'sitemap'):
            app.sitemap = {}
        if self.add_to_sitemap:
            app.sitemap['Download']=self.download_url
        return MyBlueprint.register(self, app, options, first_registration=first_registration)
    def add_handlers(self):
        self.add_static('/download', self.data_dir)
        import wpkit.linux as pylinux
        if utils.pkg_info.is_linux():
            download_func=pylinux.tools.wget_download
        else:
            import wget
            download_func=wget.download

        @self.route('/', methods=['GET'])
        def do_post_get():
            return utils.render(resources.get_default_template_string('post'))

        @self.route('/', methods=['POST'])
        def do_post_post():
            data = request.get_json()
            url = data['url']
            print('get url: %s' % url)
            download_func(url, self.data_dir)
            return 'seccess'



from wk.extra.node import *
from wk.web.resources import Sites
Site=Sites.subdir('zspt')

class CONST:
    blue = '#0088cc'
    green = '#16a085'

class Parts:
    class structure:
        searchFilePage = lambda: Site.load_frame('SearchFiles.tem')
        article = lambda: Site.load_frame('Article.tem')
        editArticlePage = lambda: Site.load_frame('EditArticle.tem')

    class plain:
        homePage = lambda: Site.load_plain('homePage.tem')
        navigator = lambda: Site.load_plain('Navigator.tem')
        articleCard = lambda: Site.load_plain('ArticleCard.tem')
        article = lambda: Site.load_plain('Article.tem')
        userHomePage=lambda :Site.load_plain('UserHomePage.tem')
        userVisitPage=lambda :Site.load_plain('UserVisitPage.tem')
        editArticlePage=lambda :Site.load_plain('EditArticle.tem')
        uploadPage=lambda :Site.load_plain('Upload.tem')
        # searchFilePage=lambda :Site.load_plain('SearchFiles.tem')
        successPage=lambda :Site.load_plain('SuccessPage.tem')
        fileListPage = lambda: Site.load_plain('FileList.tem')
        defaultSearchPage = lambda: Site.load_plain('DefaultSearchPage.tem')
        redirectPage = lambda: Site.load_plain('RedirectPage.tem')

    homePage = lambda: Site.load('homePage.tem')
    navigator=lambda :Site.load('Navigator.tem')
    articleCard=lambda :Site.load('ArticleCard.tem')
    article=lambda :Site.load('Article.tem')
    userHomePage = lambda: Site.load('UserHomePage.tem')
    userVisitPage = lambda: Site.load('UserVisitPage.tem')
    editArticlePage = lambda: Site.load('EditArticle.tem')
    uploadPage = lambda: Site.load('Upload.tem')
    # searchFilePage = lambda: Site.load('SearchFiles.tem')
    successPage = lambda: Site.load('SuccessPage.tem')
    fileListPage= lambda :Site.load('FileList.tem')
    defaultSearchPage= lambda :Site.load('DefaultSearchPage.tem')


class DefaultPageBase(Html):
    environment = dict(
        # blue='#1D93EC',
        blue='#0088cc',
        green='#16a085',
        gray='#E0E0E0'
    )

    def __init__(self):
        super().__init__()
        self.__call__([
            Head()([
                Meta(encoding='utf-8'),
                Meta(name="viewport", content="width=device-width, initial-scale=1, shrink-to-fit=no"),
                Title()(
                    Var(name="title", type='text')("知识库")
                ),
                # '<link rel="icon" href="/pkg-resource/sites/zspt/img/favicon.ico" type="image/x-icon" />',
                '<link rel="icon" href="/pkg-resource/imgs/icon/svg/知识库.svg" type="image/x-icon" />',
                QLinks.bootstrap(),
                QLinks.font_awesome(),
                QLinks.alertify(),
                QLinks.zico(),
                QScripts.jquery(),
                QScripts.popper(),

                QScripts.jwerty(),
                QLinks.font_awesome(),
                QLinks.jgrowl(),
                QScripts.jgrowl(),
                QScripts.zico(),
                QScripts.storejs(),
            ]),
            Body()([
                Var(name='body')("Add your content here"),
                QScripts.alertify(),
                QScripts.bootstrap(),
            ])
        ])

class HomePage(DefaultPageBase):
    environment = dict(
        active=0
    )
    def __init__(self):
        super().__init__()
        self.compile(
            body=[
                Parts.plain.navigator(),
                Parts.plain.homePage()
            ]
        )
class PostPage(DefaultPageBase):
    environment = dict(
        active=2,
    )
    def __init__(self):
        super().__init__()
        self.compile(
            title="Post Your Content",
            body=[
                Site.load('post.html').render(
                    navigator=Parts.plain.navigator()
                )
            ]
        )

class UploadPage(DefaultPageBase):
    environment = dict(
        active=3,
    )
    def __init__(self):
        super().__init__()
        self.compile(
            title="Post Your Content",
            body=[
                Parts.plain.navigator(),
                Parts.plain.uploadPage(),
            ]
        )

class DefaultSearchPage(DefaultPageBase):
    def __init__(self):
        super().__init__()
        self.compile(
            title='Search',
            body=[
                Parts.defaultSearchPage().render()
            ]
        )
class SearchResultPage(DefaultPageBase):
    def __init__(self):
        super().__init__()
        self.compile(
            title='知识树',
            body=[
                Parts.plain.navigator(),
                Parts.defaultSearchPage().render(),
                Var(name='result')("No Result."),
            ]
        )

class ArticleCard(Div):
    def __init__(self):
        super().__init__()
        self.__call__([
            Parts.plain.articleCard(),
        ])

class ArticlePage(DefaultPageBase):
    def __init__(self,):
        super().__init__()
        self.compile(
            body=[
                Parts.plain.navigator(),
                Parts.structure.article(),
            ]
        )

class FileSearchPageBase(DefaultPageBase):
    environment = dict(
        blue='#1D93EC',
        active=1
    )

    def __init__(self):
        super().__init__()
        self.compile(
            body=[
                Parts.plain.navigator(),
                Parts.structure.searchFilePage(),
            ]
        )

class UserHomePage(DefaultPageBase):
    def __init__(self):
        super().__init__()
        self.compile(
            body=[
                Parts.plain.navigator(),
                Parts.plain.userHomePage(),
            ]
        )
class UserVisitPage(DefaultPageBase):
    def __init__(self):
        super().__init__()
        self.compile(
            body=[
                Parts.plain.navigator(),
                Parts.plain.userVisitPage(),
            ]
        )
class EditArticlePage(DefaultPageBase):
    def __init__(self):
        super().__init__()
        self.compile(
            body=[
                Parts.plain.navigator(),
                Parts.structure.editArticlePage(),
            ]
        )
class ErrorPage(DefaultPageBase):
    def __init__(self,message='Error!'):
        super().__init__()
        self.compile(
            body=[
                Parts.plain.navigator(),
                message
            ]
        )

class NoPermissionPage(DefaultPageBase):
    def __init__(self,message='你没有权限修改该文章!'):
        super().__init__()
        self.compile(
            body=[
                Parts.plain.navigator(),
                message
            ]
        )

class PublishSuccessPage(DefaultPageBase):
    def __init__(self,message='发布成功!',link=None,link_text='前往查看'):
        super().__init__()
        self.compile(
            body=[
                Parts.plain.navigator(),
                Parts.successPage().render(message=message,link=link,link_text=link_text)
            ]
        )
class RedirectPage(DefaultPageBase):
    def __init__(self):
        super().__init__()
        self.compile(
            body=[
                Parts.plain.navigator(),
                Parts.plain.redirectPage(),
            ]
        )



from .StoreItem import *
from .ShadowStore import *
from .BranchDict import *


_BRANCH_DICT='branch_dict'
_FAKE2TRUE='fake2true'
_TRUE2FAKE='true2fake'
_BRANCH_LIST='remote_branch_list'
USER_HOME=os.path.expanduser('~')
STORE_HOME=USER_HOME+'/.store'
BRANCH_LIST_DIR=STORE_HOME+'/BranchLists'
SHADOW_STORE_HOME=STORE_HOME+'/ShadowStores'
STORE_TMP_DIR=STORE_HOME+'/tmp'
STORE_CLASS_DIR=STORE_HOME+'/Store.Class'

class Store:
    def __init__(self,path=STORE_CLASS_DIR,remote_location=None):
        cache_dir=path+'/.store.cache.store'
        if os.path.exists(path):
            shutil.rmtree(path)
        if os.path.exists(cache_dir):
            shutil.rmtree(cache_dir)
        os.makedirs(path)
        os.makedirs(cache_dir)
        self.path=path
        self.remote_location=remote_location or default_remote_location
        self.cache_dir=cache_dir
        # self.ss = ShadowStore(path + '/.shadow.main.store', remote_location, cache_dir + '/.shadow.cache.store')
        self.ss = ShadowStore(remote_location=remote_location)
        self.bd = BranchDict(remote_location=self.remote_location)
        ss = self.ss
        ss.sync_keys()
    def _new_tmpfolder(self):
        name=STORE_TMP_DIR
        return Folder(name)
    def _backup(self,path):
        folder = self._new_tmpfolder()
        folder.eat(path)
        return folder,folder._truepath(os.path.basename(path))
    def get(self,key,path=None,overwrite=False):
        ss=self.ss
        bd=self.bd
        # tmp_dir='get-tmp-'+uuid.uuid4().hex
        tmp_dir= STORE_TMP_DIR
        Folder(tmp_dir).clean()
        key=ss.key_to_branch(key)
        if not key in bd.fakes():
            bd._sync_dict()
            if not key in bd.fakes():
                raise KeyError('Key %s dose not exist.'%(key))
        true=bd.fake2true(key)
        print("Getting from %s"%(true))
        ss.get(true,path=tmp_dir,overwrite=overwrite)
        src=glob.glob(tmp_dir+'/*')[0]
        if not path:
            path='./'
        copy_fsitem(src,path,overwrite=overwrite)
        shutil.rmtree(tmp_dir)

    def set(self,key,path,recursive=False):
        key=self.ss.key_to_branch(key)
        if recursive:
            self._set_recursive(key,path,self.cache_dir)
        else:
            self._set(key,path,self.cache_dir)
    def _set_recursive(self,key,path,cache_dir):
        assert os.path.isdir(path)
        if os.path.exists(cache_dir):
            shutil.rmtree(cache_dir)
        os.makedirs(cache_dir)
        folder, path = self._backup(path)
        cache_dir = folder.openFolder('.shadows.store').path

        fake = key
        bd = self.bd
        if fake in bd.fakes():
            true=bd.fake2true(fake)
            self._upload_recursive(path=path, remote_location=self.remote_location, fake=fake, true=true,
                                   cache_dir=cache_dir)
        else:
            true=bd._generate_hash(fake)
            self._upload_recursive(path=path,remote_location=self.remote_location,fake=fake,true=true,cache_dir=cache_dir)
            bd.set(fake,true)

    def _upload_recursive(self,path,remote_location,fake,true,cache_dir):
        '''
        key is not None, delete path after it finish
        set branchdict, and remove self
        '''
        if not os.path.exists(cache_dir):
            os.makedirs(cache_dir)
        bd=self.bd
        if os.path.isfile(path):
            ss=ShadowStore(remote_location=remote_location,sync_keys=True)
            # ss=ShadowStore(cache_dir+'/.remote_branch_list.store',remote_location=remote_location,cache_dir=cache_dir+'/.cache_repos.store',sync_keys=True)
            ss.set(true,path)
        else:
            more={}
            for name in os.listdir(path):
                p=path+'/'+name
                child_cache_dir=cache_dir+"/cache_dir-"+name
                chfake=bd._concat_fakes(fake,name)
                if chfake in bd.fakes():
                    chtrue=bd.fake2true(chfake)
                else:
                    chtrue=bd._generate_hash(chfake)
                self._upload_recursive(path=p,remote_location=remote_location,fake=chfake,true=chtrue,cache_dir=child_cache_dir)
                more[name]=chtrue
            self_cache_dir=cache_dir+'/self_cache_dir'
            # ss=ShadowStore(self_cache_dir+'/.remote_branch_list.store',remote_location=remote_location,cache_dir=self_cache_dir+'/.cache_repos.store',sync_keys=True)
            ss=ShadowStore(remote_location=remote_location,sync_keys=True)
            ss.set(true,path,add_more=more)
        bd.set(fake, true)
        remove_fsitem(path)

    def _set(self,fake,path,cache_dir):
        if os.path.exists(cache_dir):
            shutil.rmtree(cache_dir)
        os.makedirs(cache_dir)
        # ss=ShadowStore(cache_dir+'/.remote_branch_list.store',remote_location=self.remote_location,cache_dir=cache_dir+'/.cache_repos.store',sync_keys=True)
        ss=ShadowStore(remote_location=self.remote_location,sync_keys=True)
        bd=self.bd
        if fake in bd.fakes():
            true=bd.fake2true(fake)
            ss.set(true,path)
        else:
            true=bd._generate_hash(fake)
            ss.set(true,path)
            bd.set(fake,true)
        print("uploading to %s"%(true))





from wpkit.web.apputils import *

class Webvpn(MyBlueprint):
    url_prefix = '/webvpn'
    nickname = 'WebVPN'
    add_to_sitemap = True
    def add_handlers(self):
        @self.route('/')
        def do_home():
            headers=request.headers
            print(headers)
            return "Hi"




from wk import FakeOS
from wpkit.web.apputils import MyBlueprint,parse_json_and_form,\
    StatusError,StatusSuccess,Status,jsonify,request,\
    get_env,Pages,send_file,get_page_template
import os

class CloudOS(FakeOS):
    def __init__(self, path=None):
        super().__init__(path)
        self.cmd_dict={}
    def execute(self,cmd):
        op=cmd['op']
        params=cmd['params']
        if op in dir(self):
            attr=self.__getattribute__(op)
        else:
            attr=self.cmd_dict[op]
        return attr(*params)
    def add_cmd(self,func):
        import functools
        @functools.wraps(func)
        def wrapper(*args,**kwargs):
            return func(*args,**kwargs)
        self.cmd_dict[func.__name__]=wrapper
        return wrapper
    def savefiles(self,files,dst):
        dst=self._truepath(dst)
        for name,file in files.items():
            fn=dst+'/'+name
            file.save(fn)
    def set_files_url_prefix(self,url_prefix):
        self.files_url_prefix=url_prefix
    def geturl(self,path=None,truepath=None):
        if truepath:
            path=truepath
        else:
            path = self._truepath(path)
        path = self._fakepath(path)
        if path == '.' or path=='/': path = ''
        path = self._standard_path(self.files_url_prefix +"/"+ path)
        return path
    def getfile(self,path):
        path=self._truepath(path)
        return send_file(path)
    def getpage(self,path):
        truepath=self._truepath(path)
        tem=get_page_template(truepath)
        print("page:",tem)
        dir=path if os.path.isdir(path) else os.path.dirname(path)
        fs = self.listdir(dir)

        map = dict({f: self.geturl(dir+'/'+f) for f in fs})
        return tem.render(map=map)



class OSServer(MyBlueprint):
    def __init__(self,url_prefix='/os',default_root_path='./',*args,**kwargs):

        self.root_path=default_root_path
        self.os=CloudOS(self.root_path)
        self.root_path=self.os.path
        self.debug=True
        super().__init__(url_prefix=url_prefix, add_to_sitemap=False, *args, **kwargs)
    def add_handlers(self):
        self.add_static('/files',self.root_path)
        @self.route('/')
        def do_root():

            return Pages.base.render()
        @self.os.add_cmd
        def getpage(path):
            path=self.os._truepath(path)
            tem=get_page_template(path)
            if tem:
                return tem.render()
        self.os.set_files_url_prefix(self.url_prefix+'/files')
        @self.route('/cmd',methods=['GET','POST'])
        @parse_json_and_form
        def do_cmd(cmd):
            print("cmd:",cmd)
            try:
                res = self.os.execute(cmd)
                if cmd['op'] in ['getfile']:
                    return res
                return jsonify(StatusSuccess(data=res))
            except:
                if self.debug:
                    raise
                else:
                    return jsonify(StatusError())
        @self.route('/upload',methods=['GET','POST'])
        @parse_json_and_form
        def do_upload(dst):
            files=request.files
            print("files:",files)
            self.os.savefiles(files,dst)
            return StatusSuccess(msg="Files saved !")

if __name__ == '__main__':
    OSServer(url_prefix='/',default_root_path='./data/os').run()





from .board import BlueBoard
from .dbserver import DBServer
from .fsserver import LocalFSServer
from .pan import BluePan
from .post_and_download import BluePostAndDownload
from .sitemap import BlueSitemap
from .welcome import BlueWelcomePage
from .static import BlueStatic
from .demo import DemoApp
from .blogserver import BlogServer
from .osserver import OSServer
from .webvpn import Webvpn


import wk


class NodeMetaClass(type):
    def __new__(cls, name, bases, attrs):
        debug=False
        if name == None:
            debug=True
        dict_attrs=['_attrs','environment']
        for name in dict_attrs:
            if name in attrs.keys():
                _attrs = attrs[name]
                tmp_dict = {}
                assert isinstance(_attrs, dict)
                for base in bases:
                    if hasattr(base, name):
                        base_attrs = getattr(base, name)
                        tmp_dict.update(**base_attrs)
                tmp_dict.update(**_attrs)
                attrs[name] = tmp_dict
        return type.__new__(cls, name, bases, attrs)


class Node(metaclass=NodeMetaClass):
    tag = 'Node'
    self_closing = False
    _attrs = {}
    _children = []
    environment={}
    def __init__(self, **kwargs):
        def preprocess(kwargs):
            cls_attr = '_class'
            if cls_attr in kwargs.keys():
                kwargs['class'] = kwargs.pop(cls_attr)
            return kwargs

        self.attrs = {}
        self.attrs.update(**self._attrs)
        self.attrs = preprocess(self.attrs)
        self.children = self._children
        kwargs = preprocess(kwargs)
        children_name = 'children'
        if children_name in kwargs.keys():
            self.children = kwargs.pop(children_name)
        self.attrs.update(**kwargs)

    def to_string(self, indent=0, indent_step=2):
        tag_and_attrs_string = ' '.join([self.tag] + ['%s="%s"' % (name, value) for name, value in self.attrs.items()])
        if self.self_closing:
            return '<{tag_and_attrs}>'.format(indent=' ' * indent, tag_and_attrs=tag_and_attrs_string)
        else:
            if len(self.children) == 1:
                "Handle such case that the child is text or Var with type of text"
                child = self.children[0]
                if isinstance(child, (str,)):
                    children_string = str(child)
                    return '<{tag_and_attrs}>{children_string}</{tag}>'.format(tag_and_attrs=tag_and_attrs_string,
                                                                               children_string=children_string,
                                                                               tag=self.tag)
                elif isinstance(child, Var) and child.attrs['type'] == 'text':
                    children_string = child.to_string(indent=indent, indent_step=indent_step)
                    return '<{tag_and_attrs}>{children_string}</{tag}>'.format(tag_and_attrs=tag_and_attrs_string,
                                                                               children_string=children_string,
                                                                               tag=self.tag)

            children_string = '\n{indent}'.format(indent=' ' * (indent + indent_step)).join([child.to_string(
                indent=indent + indent_step, indent_step=indent_step) if isinstance(child, (Node,)) else ' ' * (
                    indent + indent_step) + str(child) + '\n' for child in self.children])
            if children_string:
                return '<{tag_and_attrs}>\n{next_indent}' \
                       '{children_string}' \
                       '\n{indent}</{tag}>'.format(next_indent=' ' * (indent + indent_step), indent=' ' * indent,
                                                   tag_and_attrs=tag_and_attrs_string, children_string=children_string,
                                                   tag=self.tag)
            else:
                return '<{tag_and_attrs}></{tag}>'.format(tag_and_attrs=tag_and_attrs_string, tag=self.tag)
    def to_structure(self,indent=0,indent_step=2):
        tag_and_attrs_string = ' '.join([self.tag] + ['%s="%s"' % (name, value) for name, value in self.attrs.items()])
        if self.self_closing:
            return '<{tag_and_attrs}>'.format(indent=' ' * indent, tag_and_attrs=tag_and_attrs_string)
        else:
            if len(self.children) == 1:
                "Handle such case that the child is text or Var with type of text"
                child = self.children[0]
                if isinstance(child, (str,)):
                    children_string = str(child)
                    return '<{tag_and_attrs}>{children_string}</{tag}>'.format(tag_and_attrs=tag_and_attrs_string,
                                                                               children_string=children_string,
                                                                               tag=self.tag)
                elif isinstance(child, Var) and child.attrs['type'] == 'text':
                    children_string = child.to_structure(indent=indent, indent_step=indent_step)
                    return '<{tag_and_attrs}>{children_string}</{tag}>'.format(tag_and_attrs=tag_and_attrs_string,
                                                                               children_string=children_string,
                                                                               tag=self.tag)

            children_string = '\n{indent}'.format(indent=' ' * (indent + indent_step)).join([child.to_structure(
                indent=indent + indent_step, indent_step=indent_step) if isinstance(child, (Node,)) else ' ' * (
                    indent + indent_step) + str(child) + '\n' for child in self.children])
            if children_string:
                return '<{tag_and_attrs}>\n{next_indent}' \
                       '{children_string}' \
                       '\n{indent}</{tag}>'.format(next_indent=' ' * (indent + indent_step), indent=' ' * indent,
                                                   tag_and_attrs=tag_and_attrs_string, children_string=children_string,
                                                   tag=self.tag)
            else:
                return '<{tag_and_attrs}></{tag}>'.format(tag_and_attrs=tag_and_attrs_string, tag=self.tag)
    def __str__(self):
        return self.to_string()
        # return self.to_structure()
    def __repr__(self):
        return self.to_structure()
    def __len__(self):
        return len(self.children)

    def __call__(self, children: list = []):
        if not isinstance(children, (list,)):
            assert isinstance(children, (Node, str, Var))
            children = [children]
        self.children = children
        return self

    def to_file(self, filepath):
        with open(filepath, 'w', encoding='utf-8') as f:
            f.write(self.render())

    def compile(self, **kwargs):
        ''' replace var with specific object , if the object is a list . the insert every element in this list into self.children'''
        if not len(self.children):
            return self
        index=0
        for i in range(len(self.children)):
            child=self.children[index]
            if isinstance(child, str):
                index+=1
                continue
            if isinstance(child, Var):
                name = child.attrs['name']
                if name in kwargs.keys():
                    self.children.pop(index)
                    new_nodes=kwargs[name]
                    if not  isinstance(new_nodes,(tuple,list)):
                        new_nodes=[new_nodes]
                    for new_node in new_nodes:
                        self.children.insert(index,new_node)
                        index+=1
                else:
                    self.children[index] = child.compile(**kwargs)
                    index+=1
            else:
                self.children[index] = child.compile(**kwargs)
                index+=1
        return self

    def render(self, **kwargs):
        render_kwargs={}
        render_kwargs.update(**self.environment)
        render_kwargs.update(**kwargs)
        from jinja2 import Environment,Template

        tem = Environment().from_string(self.to_string())
        return tem.render(**render_kwargs)

class Text(Node):
    tag = 'text'
    def to_string(self, indent=0, indent_step=2):
        return self.children[0]
    def __call__(self, children: list = []):
        if not isinstance(children, (list,)):
            assert isinstance(children, ( str, ))
            children = [children]
        self.children = children
        return self

class Var(Node):
    tag = 'var'
    _attrs = dict(type='node')

    def __init__(self, name, **kwargs):
        super().__init__(name=name, **kwargs)

    def to_string(self, indent=0, indent_step=2):
        if len(self.children) == 1:
            child = self.children[0]
            if isinstance(child, (str,)):
                return str(child)
            elif isinstance(child, Var) and child.attrs['type'] == 'text':
                return child.to_string(indent=indent, indent_step=indent_step)

        children_string = '\n{indent}'.format(indent=' ' * (indent)).join([child.to_string(
            indent=indent, indent_step=indent_step) if isinstance(child, (Node,)) else ' ' * (
            indent) + str(child) + '\n' for child in self.children])
        if children_string:
            return children_string
        else:
            return ''


class Html(Node):
    tag = 'html'


class Head(Node):
    tag = 'head'


class Body(Node):
    tag = 'body'


class Header(Node):
    tag = 'header'


class Footer(Node):
    tag = 'footer'


class Link(Node):
    tag = 'link'
    self_closing = True


class Meta(Node):
    tag = 'meta'
    self_closing = True


class Title(Node):
    tag = 'title'


class Script(Node):
    tag = 'script'


class Style(Node):
    tag = 'style'


class Nav(Node):
    tag = 'nav'


class Div(Node):
    tag = 'div'


class Span(Node):
    tag = 'span'


class H1(Node):
    tag = 'h1'


class H2(Node):
    tag = 'h2'


class H3(Node):
    tag = 'h3'


class H4(Node):
    tag = 'h4'


class H5(Node):
    tag = 'h5'


class H6(Node):
    tag = 'h6'


class P(Node):
    tag = 'p'


class Table(Node):
    tag = 'table'


class Caption(Node):
    tag = 'caption'


class Thead(Node):
    tag = 'thead'


class tbody(Node):
    tag = 'tbody'


class Tr(Node):
    tag = 'tr'


class Td(Node):
    tag = 'td'


class Th(Node):
    tag = 'th'


class Ul(Node):
    tag = 'ul'


class Ol(Node):
    tag = 'ol'


class Li(Node):
    tag = 'li'


class Form(Node):
    tag = 'form'


class Textarea(Node):
    tag = 'textarea'


class Input(Node):
    tag = 'input'
    self_closing = True


class Label(Node):
    tag = 'label'


class Select(Node):
    tag = 'select'


class A(Node):
    tag = 'a'


class B(Node):
    tag = 'b'


class Strong(Node):
    tag = 'strong'


class I(Node):
    tag = 'i'


class Em(Node):
    tag = 'em'


class Strike(Node):
    tag = 'strike'


class Del(Node):
    tag = 'del'


class Hr(Node):
    tag = 'hr'
    self_closing = True


class Br(Node):
    tag = 'br'
    self_closing = True


class U(Node):
    tag = 'u'


class Img(Node):
    tag = 'img'


class Sub(Node):
    tag = 'sub'


class Sup(Node):
    tag = 'sup'


class Big(Node):
    tag = 'big'


class Small(Node):
    tag = 'small'


class Button(Node):
    tag = 'button'


def smart_update_dict(dic1={}, dic2={}):
    '''if the some-value is also a dict , then try to update only the smaller dict'''
    for k, v in dic2.items():
        if not k in dic1.keys():
            dic1[k] = v
        else:
            if isinstance(dic1[k], dict) and isinstance(dic2[k], dict):
                smart_update_dict(dic1[k], dic2[k])
            else:
                dic1[k] = v
    return dic1



from wk.extra.gitspace.StoreFolder import StoreFolder,StoreFile
from wk import get_time_formated
from wk import SimpleListFile


def main():

    # test3()
    # test5()
    # test6()
    # test7()
    # test8()
    test9()
    # test10()

    pass
def test10():
    folder = StoreFolder.openStorefolder(path='folder-deploy', remote_branch='folder-deploy',
                                         force_pull=False, overwrite=True)
def test9():
    StoreFolder.export('export',remote_branch='folder',overwrite=True)
def test8():
    folder = StoreFolder.openStorefolder(path='folder', remote_branch='folder',
                                         force_pull=False, overwrite=True)
    folder.eatStore(path='deploy')
    folder.upload()

def test7():
    file=StoreFile.openStorefile(path='msyh', remote_branch='msyh.ttf',
                                         force_pull=False, overwrite=True)
    file.upload()

def test6():
    folder = StoreFolder.openStorefolder(path='remote_branch_list', remote_branch='remote_branch_list',
                                         force_pull=True, overwrite=True)
    repo=folder.repo
    # repo.checkout_branch('remote_branch_list')
    # folder.rmfile('test.json')

    folder.upload()
def test5():
    folder=StoreFolder.openStorefolder(path='folder',remote_branch='folder',
                                       force_pull=False,overwrite=True)
    folder.status()
    add_content(folder)
    folder.upload()
    folder._pull_remote_branch_list()
    repo=folder.repo
    # repo.pull(folder.remote_location,'remote_branch_list')
    x=folder._read_remote_branch_list()
    print(x)
    folder.status()






def add_content(folder):
    dic = folder.openFiledict('test.json')
    if not dic.get('times',None):
        dic.times=[]
    time = get_time_formated()
    dic.times.append(time)
    dic._save()
def add_file(folder):
    from uuid import uuid4
    # folder=StoreFile()
    folder.newfile(uuid4().hex+'.txt')
def test1():
    folder = StoreFolder('./folder',remote_branch='folder')
    repo = folder.repo
    print(dict(repo.refs))
    add_content(folder)
    add_file(folder)
    folder.upload()

def test2():
    StoreFolder.export('export',remote_branch='folder',overwrite=True)
    import os,shutil
def test3():
    folder = StoreFile('./file', remote_branch='file')
    repo = folder.repo
    print(dict(repo.refs))
    add_content(folder)
    folder.upload()
def test4():
    StoreFile.export('export', remote_branch='file', overwrite=True)


if __name__ == '__main__':
    main()
# test1()
# test2()

# test3()
# test4()


# ref='refs/remotes/origin/bgs-1'
# repo.checkout_branch(ref)
#
# a=folder.openFieldict('test.json')
# a.time=get_time_formated()
# folder.upload()
